package DataFileConverters;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;

public class CoNNL03 {

	static void printTabSeparatedLine(BufferedWriter out, String[] words, int numTokens) throws IOException{
		int idx;
		for (idx = 0; idx < numTokens; idx++) {
			out.write(words[idx]+"\t");
		}
		out.newLine();
	}
	
	static void convertFileFormat(String inFile, String outFile) throws IOException {
		BufferedReader in = new BufferedReader(new InputStreamReader(
				new FileInputStream(inFile), "8859_2"));
		BufferedWriter out = new BufferedWriter(new FileWriter(outFile));

		String[] tokens;

		int maxSentenceLength = 250;
		String[] words = new String[maxSentenceLength];
		String[] posTags = new String[maxSentenceLength];
		String[] nerTags = new String[maxSentenceLength];

		int tokIdx = 0;

		String line = in.readLine();
		while (line != null) {
			
			if (line.equals("")) {
				// print instance to output file
				printTabSeparatedLine(out, words, tokIdx);
				printTabSeparatedLine(out, posTags, tokIdx);
				printTabSeparatedLine(out, nerTags, tokIdx);

				// start next instance
				tokIdx = 0;
				line = in.readLine();
				continue;
			}

			// parse next tokens in current instance
			tokens = line.split(" ");
			if (tokens.length != 4) {
				throw new RuntimeException("Wrong tokens num for line: " + line);
			}
			words[tokIdx] = tokens[0];
			posTags[tokIdx] = tokens[1];
			nerTags[tokIdx] = tokens[3];
			tokIdx++;

			line = in.readLine();
		}

		in.close();
		out.flush();
		out.close();
	}

	/**
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		if (args.length < 2) {
			System.out.println("Usage: <src file> <target file>");
			return;
		}
		String inFile = args[0];
		String outFile = args[1];

		System.out.println("Converting: " + inFile + " to: " + outFile);
		convertFileFormat(inFile, outFile);
	}
}
