package Main;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

public class McNemarParse {

	public static void main(String[] args) throws Exception {
		McNemarParse inst = new McNemarParse();
		inst.run(args);
	}

	private void run(String[] args) throws IOException {
		if (args.length < 2) {
			System.out.println("Usage: file1 file2");
			return;
		}

		String file1 = args[0];
		String file2 = args[1];

		BufferedReader in1 = new BufferedReader(new InputStreamReader(
				new FileInputStream(file1), "8859_2"));
		BufferedReader in2 = new BufferedReader(new InputStreamReader(
				new FileInputStream(file2), "8859_2"));

		String line1 = in1.readLine();
		String line2 = in2.readLine();
		
		in1.close();
		in2.close();
		
		if ( line1.length() != line2.length() ){
			System.out.println("Unequal num of tags in files.");
			return;			
		}
		
		char[] chars1 = line1.toCharArray();
		char[] chars2 = line2.toCharArray();
		int numOfTags = chars1.length;
		
		// Now count the a,b,c,d parameters relevant for the McNemar's test
		//
		//			POS		NEG
		//	POS		a		b
		//	NEG		c		d
		//
		int a=0, b=0, c=0, d=0;
		for ( int i=0; i<numOfTags; i++ ){
			char c1 = chars1[i];
			char c2 = chars2[i];
			if (c1=='T' && c2=='T'){
				a++;
			}
			else if (c1=='T' && c2=='F'){
				b++;
			}
			else if (c1=='F' && c2=='T'){
				c++;
			}
			else if (c1=='F' && c2=='F'){
				d++;
			}
			else{
				System.out.println("Unknown type is results file.");
				System.out.println("c1="+c1);
				System.out.println("c2="+c2);
				return;	
			}
		}
		
		double abs_bminc = Math.abs(b-c);
		double numerator = (abs_bminc-1)*(abs_bminc-1);
		double denominator = b+c;
		double X2 = numerator/denominator;
		
		System.out.println("\tPOS\tNEG");
		System.out.println("POS\t"+a+"\t"+b);
		System.out.println("NEG\t"+c+"\t"+d);
		System.out.println();
		System.out.println("X2="+X2);
		
	}

}
