package Main;

import java.util.Iterator;
import java.util.LinkedList;

import struct.alg.KBestBinaryReductionCWUpdator;
import struct.alg.KBestMiraUpdator;
import struct.alg.OneBestBinaryReductionCWUpdator;
import struct.alg.OnlineUpdator;
import struct.alg.PerceptronUpdator;
import struct.sequence.CWSequencePredictor;
import struct.sequence.ChunkerDataManager;
import struct.sequence.NeDataManager;
import struct.sequence.SequenceDataManager;
import struct.sequence.SequenceInstance;
import struct.sequence.SequencePredictor;
import struct.sequence.SequencePredictor.PredictCommitteeType;

public class tagger {

	String runmode;
	String dataManagerType;
	String trainFileName;
	String develFileName;
	String testFileName;
	String predictByCommittee;
	String modelFileName;
	String posNegResFileName;
	String learnerType;
	String cwUpdaterType;
	String trainIters;
	String hyperParam1;
	String cwKBest;
	String cwPhiList;
	double[] cwPhiListVals;
	String cwInitSigma;
	String miraKBest;
	String averageParams = "true";
	String committeeType = "kBestViterbi";
	String cvFolds;

	String compareConfAt;
	LinkedList<String> confEstimatorList = new LinkedList<String>();
	String[] confEstimators;

	// args expected as: type1=value1 type2=value2 ...
	// retval = continue app or not
	public boolean parseArgs(String[] args) {
		if (args.length == 0) {
			System.out.println("Options:");
			System.out
					.println("mode=[train, test, cv, inc_train, cv_inc_train, tune_cw_phi, compareconfmeasures]");
			System.out.println("datamanager=[ner, chunk]");
			System.out.println("trainfile=[filename]");
			System.out.println("develfile=[filename]");
			System.out.println("testfile=[filename]");
			System.out.println("modelfile=[filename]");
			System.out.println("learner=[mira, cw, perceptron]");
			System.out
					.println("cwupdater=[diagvardrop, diagvarproj, arowdrop, arowproj, arowexact]");
			System.out
					.println("predictbycommittee=[KDrawsByStddev*x*K, KDrawsByConst*x*K] K=size of committee, x=Fixed StdDev for drawing");
			System.out.println("trainiters=[n] default=10");
			System.out.println("cwphi=[phi] default=0.1");
			System.out.println("cwkbest=[K] default=1");
			System.out.println("cwphilist=[val1*val2*...*valn]");
			System.out.println("cwinitsigma=[Init sigma] default=1.0");
			System.out.println("mirakbest=[K] default=5");
			System.out.println("mirac=[c] default= +Inf");
			System.out.println("averageparams=[true, false] default=true");
			System.out
					.println("committeetype=[kBestViterbi, kDrawsByConst, kDrawsByStdDev] default=kBestViterbi");

			// Cross validation
			System.out.println("cvfolds=[n] default=5");

			// Results file for McNemar test
			System.out.println("posnegresfile=[filename]");

			// don't continue running
			return false;
		}

		String[] params;
		for (int i = 0; i < args.length; i++) {
			params = args[i].split("=");
			String type = params[0];
			String value = params[1];

			if (type.equals("mode")) {
				runmode = value;

				// set default Cross validation folds
				boolean isCv = (runmode.equals("cv") || runmode
						.equals("cv_inc_train"));
				if (isCv && cvFolds == null) {
					cvFolds = "5";
				}

				// Set default training iterations
				boolean isTrain = (runmode.equals("train") || runmode
						.equals("tune_cw_phi"));
				if ((isTrain || isCv) && trainIters == null) {
					trainIters = "10";
				}

			} else if (type.equals("datamanager")) {
				dataManagerType = value;
			} else if (type.equals("trainfile")) {
				trainFileName = value;
			} else if (type.equals("develfile")) {
				develFileName = value;
			} else if (type.equals("testfile")) {
				testFileName = value;
			} else if (type.equals("modelfile")) {
				modelFileName = value;
			} else if (type.equals("posnegresfile")) {
				posNegResFileName = value;
			} else if (type.equals("learner")) {
				learnerType = value;

				// set default CW parameters
				// set default CW Phi value
				if (learnerType.equals("cw") && hyperParam1 == null) {
					hyperParam1 = "0.1";
				}

				// set default K for K-Best CW
				if (learnerType.equals("cw") && cwKBest == null) {
					cwKBest = "1";
				}

				// set default CW Init sigma value
				if (learnerType.equals("cw") && cwInitSigma == null) {
					cwInitSigma = "1.0";
				}

				// set default MIRA K
				if (learnerType.equals("mira") && miraKBest == null) {
					miraKBest = "5";
				}
			} else if (type.equals("cwupdater")) {
				cwUpdaterType = value;
			} else if (type.equals("compareconfat")) {
				compareConfAt = value;
			} else if (type.equals("confmethod")) {
				confEstimatorList.addLast(value);
			} else if (type.equals("trainiters")) {
				trainIters = value;
			} else if (type.equals("predictbycommittee")) {
				predictByCommittee = value;
			} else if (type.equals("cwphi")) {
				hyperParam1 = value;
			} else if (type.equals("mirac")) {
				hyperParam1 = value;
			} else if (type.equals("cwkbest")) {
				cwKBest = value;
			} else if (type.equals("cwphilist")) {
				cwPhiList = value;
			} else if (type.equals("cwinitsigma")) {
				cwInitSigma = value;
			} else if (type.equals("mirakbest")) {
				miraKBest = value;
			} else if (type.equals("averageparams")) {
				averageParams = value;
			} else if (type.equals("committeetype")) {
				committeeType = value;
			} else if (type.equals("cvfolds")) {
				cvFolds = value;
			} else {
				throw new RuntimeException("Unknown argument: " + type);
			}
		}

		// print arguments:
		System.out.println("Running:");
		if (runmode != null)
			System.out.println("mode=" + runmode);
		if (dataManagerType != null)
			System.out.println("datamanager=" + dataManagerType);
		if (trainFileName != null)
			System.out.println("trainfile=" + trainFileName);
		if (develFileName != null)
			System.out.println("develfile=" + develFileName);
		if (testFileName != null)
			System.out.println("testfile=" + testFileName);
		if (modelFileName != null)
			System.out.println("modelfile=" + modelFileName);
		if (learnerType != null)
			System.out.println("learner=" + learnerType);
		if (cwUpdaterType != null)
			System.out.println("cwupdater=" + cwUpdaterType);
		if (predictByCommittee != null)
			System.out.println("predictbycommittee=" + predictByCommittee);
		if (compareConfAt != null)
			System.out.println("compareconfat=" + compareConfAt);
		if (confEstimatorList.isEmpty() == false) {
			confEstimators = new String[confEstimatorList.size()];
			int idx = 0;
			System.out.print("confmethod= ");
			Iterator<String> iter = confEstimatorList.iterator();
			while (iter.hasNext()) {
				String estimator = iter.next();
				System.out.print(estimator + ", ");
				confEstimators[idx++] = estimator;
			}
			System.out.println();
		}
		if (hyperParam1 != null)
			System.out.println("hyperParam1=" + hyperParam1);
		if (cwKBest != null)
			System.out.println("cwkbest=" + cwKBest);
		if (cwPhiList != null) {
			System.out.println("cwphilist=" + cwPhiList);
			String[] phiValsStr = cwPhiList.split("\\*");
			cwPhiListVals = new double[phiValsStr.length];
			for (int i = 0; i < cwPhiListVals.length; i++) {
				cwPhiListVals[i] = Double.parseDouble(phiValsStr[i]);
			}
		}
		if (cwInitSigma != null)
			System.out.println("cwinitsigma=" + cwInitSigma);
		if (miraKBest != null)
			System.out.println("mirakbest=" + miraKBest);
		if (averageParams != null)
			System.out.println("averageparams=" + averageParams);
		if (committeeType != null)
			System.out.println("committeetype=" + committeeType);
		if (cvFolds != null)
			System.out.println("cvfolds=" + cvFolds);
		if (posNegResFileName != null)
			System.out.println("posnegresfile=" + posNegResFileName);

		return true;
	}

	OnlineUpdator resolveUpdatorByArgs() {
		OnlineUpdator updator = null;
		if (learnerType.equals("cw")) {
			double cwPhiVal = Double.valueOf(hyperParam1);
			int k = Integer.valueOf(cwKBest);
			if (k == 1) {
				updator = new OneBestBinaryReductionCWUpdator(cwUpdaterType,
						cwPhiVal);
			} else {
				updator = new KBestBinaryReductionCWUpdator(cwUpdaterType,
						cwPhiVal, k);
			}
		} else if (learnerType.equals("mira")) {
			int K = Integer.parseInt(miraKBest);
			double miraCVal;
			if (hyperParam1 != null) {
				miraCVal = Double.valueOf(hyperParam1);
			} else {
				miraCVal = Double.POSITIVE_INFINITY;
			}
			updator = new KBestMiraUpdator(K, miraCVal);
		} else if (learnerType.equals("perceptron")) {
			updator = new PerceptronUpdator();
		}
		return updator;
	}

	SequencePredictor resolvePredictorByArgs(int numFeats) {
		SequencePredictor predictor = null;
		if (learnerType.equals("cw")) {
			double initSigmaValue = Double.parseDouble(cwInitSigma);
			PredictCommitteeType commType = SequencePredictor
					.resolveCommitteeType(committeeType);
			predictor = new CWSequencePredictor(numFeats, initSigmaValue,
					commType);
		} else if (learnerType.equals("mira")
				|| learnerType.equals("perceptron")) {
			predictor = new SequencePredictor(numFeats);
		}
		return predictor;
	}

	public SequenceDataManager resolveDataManagerByArgs() {
		if (dataManagerType.equals("ner")) {
			return new NeDataManager();
		}
		if (dataManagerType.equals("chunk")) {
			return new ChunkerDataManager();
		}
		return null;
	}

	public void run(String[] args) throws Exception {
		if (parseArgs(args) == false) {
			return;
		}

		if (runmode.equals("cv")) {
			int numIters = Integer.parseInt(trainIters);
			int cvFoldsNum = Integer.parseInt(cvFolds);
			boolean bAverageParams = Boolean.parseBoolean(averageParams);
			CrossValidation crossValidation = new CrossValidation(this);
			crossValidation.crossValidation(trainFileName, posNegResFileName,
					cvFoldsNum, numIters, bAverageParams, predictByCommittee);

		} else if (runmode.equals("train")) {
			int numIters = Integer.parseInt(trainIters);
			boolean bAverageParams = Boolean.parseBoolean(averageParams);
			TrainAndTest trainAndTest = new TrainAndTest(this);
			trainAndTest.runTrain(trainFileName, modelFileName, numIters,
					bAverageParams);
		} else if (runmode.equals("test")) {

			boolean appendToResFile = false;
			TrainAndTest trainAndTest = new TrainAndTest(this);
			trainAndTest.runTest(testFileName, modelFileName, null,
					posNegResFileName, appendToResFile, predictByCommittee);

		} else if (runmode.equals("inc_train")) {
			// run training using 1,2,...maxIters iterations and evaluate
			// performance after each iteration
			boolean appendToResFile = false;
			TrainAndTest trainAndTest = new TrainAndTest(this);
			int maxNumOfTrainingIters = 10;
			boolean bAverageParams = Boolean.parseBoolean(averageParams);
			trainAndTest.incrementalIterationsTrainAndTest(trainFileName,
					modelFileName, testFileName, maxNumOfTrainingIters,
					bAverageParams, null, posNegResFileName, appendToResFile,
					predictByCommittee);

		} else if (runmode.equals("cv_inc_train")) {
			// CV + incremental training
			int maxNumOfTrainingIters = 10;
			boolean bAverageParams = Boolean.parseBoolean(averageParams);
			int cvFoldsNum = Integer.parseInt(cvFolds);
			CrossValidation crossValidation = new CrossValidation(this);
			crossValidation.crossValidationIncrementalTraining(trainFileName,
					posNegResFileName, cvFoldsNum, bAverageParams,
					maxNumOfTrainingIters, predictByCommittee);

		} else if (runmode.equals("compareconfmeasures")) {
			SequencePredictor predictor = null;
			predictor = resolvePredictorByArgs(0);
			predictor.loadModel(modelFileName);

			SequenceDataManager manager = resolveDataManagerByArgs();

			manager.dataAlphabet = SequenceInstance.getDataAlphabet();
			manager.tagAlphabet = SequenceInstance.getTagAlphabet();
			manager.closeAlphabets();

			int numFeats = manager.dataAlphabet.size();
			System.out.println("Feature Dimension: " + numFeats);

			boolean createFeaturesFile = false;
			boolean createCrfFile = false;
			SequenceInstance[] testData = manager.readData(testFileName,
					createFeaturesFile, createCrfFile);

			CompareConfMeasures compareConfMeasures = new CompareConfMeasures(
					this);
			compareConfMeasures.compareConfMeasures(testData, predictor,
					null/* testFileName + ".feats" */, Integer
							.parseInt(compareConfAt), confEstimators);
		}
	}

	public static void main(String[] args) throws Exception {
		tagger tagger = new tagger();
		tagger.run(args);
	}

}
