import struct.alg.KBestMiraUpdator;
import struct.alg.OnlineLearner;
import struct.alg.OnlineUpdator;
import struct.sequence.NeDataManager;
import struct.sequence.SequenceDataManager;
import struct.sequence.SequenceEvaluator;
import struct.sequence.SequenceInstance;
import struct.sequence.SequencePredictor;

public class ReutersTagger {
	public static void main(String[] args) throws Exception {

		// Reuters and Chunking main is the same, so avoid confusion...
		if ( args.length > 1 ){
			return;
		}
		
		String runmode = args[0];
		String trainFileName;
		String testFileName;
		String modelFileName;
		String params;

		if (runmode.equals("train")) {

			trainFileName = args[1];
			testFileName = args[2];
			modelFileName = args[3];
			if (args.length < 5) {
				params = "u:MMMN_OPTIMAL C:0.000476 k:0 q:HILDRETH_SEQ_ALG_B00 n:1e-1 s:5e-4 m:2"; // default...
			} else {
				params = "";
				for (int i = 4; i < args.length; i++) {
					params += args[i] + " ";
				}
			}

			SequenceDataManager manager = new NeDataManager();

			boolean createUnsupportedFeatures = false;
			manager.createAlphabets(trainFileName, createUnsupportedFeatures);
			manager.closeAlphabets();

			int numFeats = manager.dataAlphabet.size();
			System.out.println("Feature Dimension: " + numFeats);

			boolean createFeaturesFile = true;
			SequenceInstance[] trainingData = manager.readData(trainFileName,
					createFeaturesFile);
			SequenceInstance[] develData = manager.readData(testFileName,
					createFeaturesFile);

			int numIters = 10;

			OnlineLearner learner = new OnlineLearner();
			SequencePredictor predictor = new SequencePredictor(numFeats);
			// OnlineUpdator updator = new PerceptronUpdator();
			// OnlineUpdator updator = new QPUpdators(params,
			// manager.tagAlphabet);
			OnlineUpdator updator = new KBestMiraUpdator(5);

			learner.train(trainingData, updator, predictor, numIters,
					trainFileName + ".feats");

			// SequenceEvaluator seqEvaluator = new SequenceEvaluator(manager
			// .getTagAlphabet());
			//
			// learner.trainAndEvaluate(trainingData, develData, updator,
			// predictor, numIters, trainFileName + ".feats", testFileName
			// + ".feats", seqEvaluator);

			predictor.saveModel(modelFileName);
		}

		else {

			testFileName = args[1];
			modelFileName = args[2];

			SequencePredictor predictor = new SequencePredictor(0);
			predictor.loadModel(modelFileName);

			SequenceDataManager manager = new NeDataManager();

			manager.dataAlphabet = SequenceInstance.getDataAlphabet();
			manager.tagAlphabet = SequenceInstance.getTagAlphabet();
			manager.closeAlphabets();

			int numFeats = manager.dataAlphabet.size();
			System.out.println("Feature Dimension: " + numFeats);

			boolean createFeaturesFile = true;
			SequenceInstance[] develData = manager.readData(testFileName,
					createFeaturesFile);

			SequenceEvaluator eval = new SequenceEvaluator(manager
					.getTagAlphabet());

			eval.evaluate(develData, predictor, testFileName + ".feats");

		}

	}
}
