package struct.alg;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

import struct.types.Evaluator;
import struct.types.Features;
import struct.types.Instance;

public class OnlineLearner implements StructuredLearner {

	public void train(Instance[] training, OnlineUpdator update,
			Predictor predictor, int numIters, String featureFile)
			throws IOException {
		train(training, update, predictor, numIters, featureFile, true);
	}

	public void train(Instance[] training, OnlineUpdator update,
			Predictor predictor, int numIters, String featureFile,
			boolean avgParams) throws IOException {

		System.out.println("Training instances: " + training.length
				+ ", Iterations: " + numIters);
		for (int i = 0; i < numIters; i++) {

			// System.out.println("==========================");
			// System.out.println("Training iteration: " + i);
			// System.out.println("==========================");
			// long start = System.currentTimeMillis();

			ObjectInputStream in = (featureFile != null) ? new ObjectInputStream(
					new FileInputStream(featureFile))
					: null;

			for (int j = 0; j < training.length; j++) {
				// System.out.print(".");

				Instance inst = training[j];
				Features feats;
				if (in != null) {
					feats = training[j].getFeatures(in);
				} else {
					feats = training[j].getFeatures();
				}

				double avg_upd = (double) (numIters * training.length - ((training.length * i) + j));

				update.update(inst, feats, predictor, avg_upd);

			}

			if (in != null)
				in.close();

			// long end = System.currentTimeMillis();
			// System.out.println("Training took: " + (end - start));
		}

		if (avgParams) {
			System.out.println("Averaging parameters...");
			predictor.averageWeights(numIters * training.length);
		}

	}

	public void trainAndEvaluate(Instance[] training, Instance[] testing,
			OnlineUpdator update, Predictor predictor, int numIters,
			String trainingFeatureFile, String testingFeatureFile,
			Evaluator eval) throws IOException {

		trainAndEvaluate(training, testing, update, predictor, numIters,
				trainingFeatureFile, testingFeatureFile, eval, true);

	}

	public void trainAndEvaluate(Instance[] training, Instance[] testing,
			OnlineUpdator update, Predictor predictor, int numIters,
			String trainingFeatureFile, String testingFeatureFile,
			Evaluator eval, boolean avgParams) throws IOException {

		train(training, update, predictor, numIters, trainingFeatureFile,
				avgParams);

		/*
		 * for (int iterNum = 0; iterNum < numIters; iterNum++) { //
		 * System.out.print(i + " ");
		 * System.out.println("==========================");
		 * System.out.println("Training iteration: " + iterNum);
		 * System.out.println("=========================="); long start =
		 * System.currentTimeMillis();
		 * 
		 * ObjectInputStream in = new ObjectInputStream(new FileInputStream(
		 * trainingFeatureFile));
		 * 
		 * for (int instIdx = 0; instIdx < training.length; instIdx++) { //
		 * System.out.print(".");
		 * 
		 * Instance inst = training[instIdx]; Features feats =
		 * training[instIdx].getFeatures(in);
		 * 
		 * double avg_upd = (double) (numIters training.length -
		 * (training.length * ((iterNum + 1) - 1) + (instIdx + 1)) + 1);
		 * 
		 * update.update(inst, feats, predictor, avg_upd);
		 * 
		 * }
		 * 
		 * // System.out.println("");
		 * 
		 * in.close();
		 * 
		 * long end = System.currentTimeMillis();
		 * System.out.println("Training took: " + (end - start)); //
		 * System.out.println("Training"); //
		 * eval.evaluate(training,predictor,trainingFeatureFile); //
		 * System.out.println("Testing"); //
		 * eval.evaluate(testing,predictor,testingFeatureFile); }
		 * System.out.println("");
		 * 
		 * if (avgParams) { System.out.println("Averaging parameters...");
		 * predictor.averageWeights(numIters * training.length); }
		 */
		System.out.println("==========================");
		System.out.println("Final Performance.");
		System.out.println("==========================");
		System.out.println("Training");
		eval.evaluate(training, predictor, trainingFeatureFile);
		System.out.println("Testing");
		eval.evaluate(testing, predictor, testingFeatureFile);

	}

}
