package struct.alg;

import struct.types.FeatureVector;
import struct.types.Features;
import struct.types.Instance;
import struct.types.Label;
import struct.types.Prediction;

public class PerceptronUpdator implements OnlineUpdator {

	public void update(Instance inst, Features feats, Predictor predictor,
			double avg_upd) {

		Prediction pred = predictor.decode(inst, feats);
		Label label = pred.getBestLabel();
		FeatureVector guessed_fv = label.getFeatureVectorRepresentation();
		FeatureVector corr_fv = inst.getLabel()
				.getFeatureVectorRepresentation();

		if (predictor.score(corr_fv) > predictor.score(guessed_fv)) {
			System.out.print("+");
			return;
		}

		double[] weights = predictor.weights;
		double[] avg_weights = predictor.avg_weights;

		for (FeatureVector curr = corr_fv; curr.next != null; curr = curr.next) {
			if (curr.index >= 0) {
				weights[curr.index] += curr.value;
				avg_weights[curr.index] += avg_upd * curr.value;
			}
		}

		for (FeatureVector curr = guessed_fv; curr.next != null; curr = curr.next) {
			if (curr.index >= 0) {
				weights[curr.index] -= curr.value;
				avg_weights[curr.index] -= avg_upd * curr.value;
			}
		}

	}

}
