package struct.alg;

import struct.types.FeatureVector;
import struct.types.Features;
import struct.types.Instance;
import struct.types.Prediction;

public abstract class Predictor {

	public double[] weights;
	public double[] avg_weights;

	public Predictor(int dimensions) {
		weights = new double[dimensions];
		avg_weights = new double[dimensions];
		for (int i = 0; i < dimensions; i++) {
			weights[i] = 0.0;
			avg_weights[i] = 0.0;
		}
	}

	public void averageWeights(int factor) {
		for (int i = 0; i < avg_weights.length; i++)
			weights[i] = avg_weights[i] / factor;
	}

	public double score(FeatureVector fv) {
		return score(fv, this.weights);
	}

	public double score(FeatureVector fv, double[] weightsVector) {
		double score = 0.0;
		for (FeatureVector curr = fv; curr != null; curr = curr.next) {
			if (curr.index >= 0)
				score += weightsVector[curr.index] * curr.value;
		}
		if ( Double.isInfinite(score) || Double.isNaN(score )){
			System.out.println("Score is Inf or NaN");
		}
		return score;
	}

	public abstract Prediction decode(Instance inst, Features feats);

	public abstract Prediction decode(Instance inst, Features feats, int K);

}
