package struct.relation;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectOutputStream;
import java.util.LinkedList;

import struct.types.Alphabet;
import struct.types.DataManager;
import struct.types.FeatureVector;

public class RelationExtractorDataManager implements DataManager {

    public Alphabet dataAlphabet;	

    public RelationExtractorDataManager()  {
	dataAlphabet = new Alphabet();
    }

    public Alphabet getDataAlphabet() { return dataAlphabet; }

    public RelationExtractorInstance[] readData(String file) throws IOException {
	return readData(file,true);
    }

    public RelationExtractorInstance[] readData(String file, boolean createFeatureFile) throws IOException {
		
	BufferedReader in =
	    new BufferedReader(new InputStreamReader(new FileInputStream(file),"8859_2"));
		
	LinkedList lt = new LinkedList();

	ObjectOutputStream out = 
	    createFeatureFile ? new ObjectOutputStream(new FileOutputStream(file+".feats")) : null;
		
	BufferedWriter outp = new BufferedWriter(new FileWriter(file+".pairs"));
		
	RelationExtractorInstanceGenerator.outp = outp;

	System.out.println("Creating feature vectors and/or forests ...");
	int num = 0;
	// instance[0] - is gold standard full labels
	// instance[1] - is array of inputs
	// do it once to initialize
	RelationExtractorInstanceGenerator.generate(in);
	String[][] instance = RelationExtractorInstanceGenerator.generate(in);
	while(instance != null) {
	    num += instance[1].length;
	    String[] labs = instance[0];
	    String[] bin_labs = new String[instance[1].length];
	    String[] e1 = new String[instance[1].length];
	    String[] e2 = new String[instance[1].length];
	    String[] t1 = new String[instance[1].length];
	    String[] t2 = new String[instance[1].length];
	    String[] filenm = new String[instance[1].length];
	    FeatureVector[] n_fv = new FeatureVector[instance[1].length];
	    FeatureVector[] y_fv = new FeatureVector[instance[1].length];
	    FeatureVector[] corr_fv = new FeatureVector[instance[1].length];
	    for(int i = 0; i < instance[1].length; i++) {
		String inpt = instance[1][i];
		String[] toks = inpt.split("\t");
		bin_labs[i] = toks[0];
		e1[i] = toks[toks.length-3]; e2[i] = toks[toks.length-2];
		filenm[i] = toks[toks.length-1];
		t1[i] = toks[3]; t2[i] = toks[4];
		n_fv[i] = createFeatureVector(inpt,"N",new FeatureVector(-1,-1.0,null));
		y_fv[i] = createFeatureVector(inpt,"Y",new FeatureVector(-1,-1.0,null));
		corr_fv[i] = createFeatureVector(inpt,bin_labs[i],new FeatureVector(-1,-1.0,null));
	    }
	    RelationExtractorInstance rei = 
		new RelationExtractorInstance(new RelationExtractorInput(e1,e2,t1,t2,filenm,n_fv,y_fv),
					      new RelationExtractorLabel(labs,bin_labs,corr_fv),
					      instance[1]);

	    lt.add(rei);

	    instance = RelationExtractorInstanceGenerator.generate(in);
	}
	
	System.out.println("Number of binary relations: " + num);

	RelationExtractorInstance[] si = new RelationExtractorInstance[lt.size()];
	for(int i = 0; i < si.length; i++) {
	    si[i] = (RelationExtractorInstance)lt.get(i);
	}

	if(createFeatureFile)
	    out.close();
		
	outp.close();
	return si;
    }

    public void createAlphabets(String file) throws IOException {
	createAlphabets(file,false);
    }

    public void createAlphabets(String file, boolean createUnsupported) throws IOException {
	System.out.print("Creating Data Alphabet ... ");
	System.out.println("done.");
    }
	


    private FeatureVector createFeatureVector(String inpt, String label, FeatureVector fv) {
	String[] features = inpt.split ("\t");
	String[] toks = new String[features.length-1];
	for(int i = 1; i < features.length; i++)
	    toks[i-1] = features[i];

	int len = toks.length;
	
	String e1W = toks[0];
	String e2W = toks[1];
	String e1T = toks[2];
	String e2T = toks[3];
	String eT = e1T+e2T;
	int numB = Integer.parseInt(toks[4]);
	String[] wordsB = new String[numB];
	String[] posB = new String[numB];
	for(int i = 0; i < numB; i+=2) {
	    wordsB[i] = toks[4+i+1];
	    posB[i] = toks[4+i+2];
	}
		
	String dist = "";
	if(numB > 10) {
	    fv = fv.add(label+"_"+"DIST=inf",1.0,dataAlphabet);
	    dist = "DIST=inf";
	}
	if(numB <= 10) {
	    fv = fv.add(label+"_"+"DIST=10",1.0,dataAlphabet);
	    dist = "DIST=10";
	}
	if(numB <= 8) {
	    fv = fv.add(label+"_"+"DIST=8",1.0,dataAlphabet);
	    dist = "DIST=8";			
	}
	if(numB <= 5) {
	    fv = fv.add(label+"_"+"DIST=5",1.0,dataAlphabet);
	    dist = "DIST=5";
	}
	if(numB <= 3) {
	    fv = fv.add(label+"_"+"DIST=3",1.0,dataAlphabet);
	    dist = "DIST=3";
	}
	if(numB <= 2) {
	    fv = fv.add(label+"_"+"DIST=2",1.0,dataAlphabet);
	    dist = "DIST=2";
	}
	if(numB <= 1) {
	    fv = fv.add(label+"_"+"DIST=1",1.0,dataAlphabet);
	    dist = "DIST=1";
	}
	if(numB <= 0) {
	    fv = fv.add(label+"_"+"DIST=0",1.0,dataAlphabet);
	    dist = "DIST=0";
	}
	
	String[] ew1s = e1W.toLowerCase().split(" ");
	String[] ew2s = e2W.toLowerCase().split(" ");
	for(int i = 0; i < ew1s.length; i++) {
	    fv = fv.add(label+"_"+"AUNITEXT="+ew1s[i],1.0,dataAlphabet);
	}
	for(int i = 0; i < ew2s.length; i++) {
	    fv = fv.add(label+"_"+"BUNITEXT="+ew2s[i],1.0,dataAlphabet);
	}
	for(int i = 0; i < ew1s.length; i++) {
	    for(int j = 0; j < ew2s.length; j++) {
		fv = fv.add(label+"_"+"BITEXT="+ew1s[i]+"__"+ew2s[j],1.0,dataAlphabet);
		fv = fv.add(label+"_"+"BITEXT="+ew2s[j]+"__"+ew1s[i],1.0,dataAlphabet);
	    }
	}
	
	for(int i = 0; i < wordsB.length; i++) {
	    if(posB[i] == null)
		continue;
	    fv = fv.add(label+"_"+"POSB="+posB[i],1.0,dataAlphabet);
	    fv = fv.add(label+"_"+"POSB="+posB[i]+"__"+dist,1.0,dataAlphabet);
	    fv = fv.add(label+"_"+"ETPOSB="+posB[i]+"__"+eT,1.0,dataAlphabet);
	    fv = fv.add(label+"_"+"E1POSB="+posB[i]+"__"+e1T,1.0,dataAlphabet);
	    fv = fv.add(label+"_"+"E2POSB="+posB[i]+"__"+e2T,1.0,dataAlphabet);
	    if(i > 0 && posB[i-1] != null) {
		fv = fv.add(label+"_"+"BIPOSB="+posB[i-1]+"__"+posB[i],1.0,dataAlphabet);
	    }
	    for(int j = i+1; j < wordsB.length; j++) {
		if(posB[j] != null)
		    fv = fv.add(label+"_"+"PIPOSB="+posB[i]+"__"+posB[j],1.0,dataAlphabet);
	    }
	}
	
	for(int i = 5+(numB*2); i < len-3; i++) {
	    fv = fv.add(label+"_"+toks[i],1.0,dataAlphabet);
	}

	return fv;
	//return FeatureVector.twoNormalize(fv);
    }
    
    public void closeAlphabets() {
	dataAlphabet.stopGrowth();
    }

}
