package struct.relation;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;

import struct.alg.Predictor;
import struct.types.Evaluator;
import struct.types.Features;
import struct.types.Instance;

public class RelationExtractorEvaluator implements Evaluator {

    public void evaluate(Instance[] data, Predictor pred) {
	System.out.println("Not implemented.");
    }

    public void evaluate(Instance[] data, Predictor predictor, String featureFile) throws IOException {
	int guessed_bin = 0; int corr_bin = 0; int total_bin = 0;
	int guessed = 0; int corr = 0; int total = 0;

	for(int k = 0; k < data.length; k++) {

	    Instance inst = data[k];
	    Features feats = data[k].getFeatures();

	    RelationExtractorPrediction prediction = 
		(RelationExtractorPrediction)predictor.decode(inst,feats);

	    RelationExtractorLabel pred = prediction.getBestLabel();
	    RelationExtractorLabel act = (RelationExtractorLabel)inst.getLabel();

	    String[] pred_bin_labels = pred.bin_labels;
	    String[] act_bin_labels = act.bin_labels;
	    for(int i = 0; i < pred_bin_labels.length; i++) {
		if(pred_bin_labels[i].equals("Y"))
		    guessed_bin++;
		if(act_bin_labels[i].equals("Y"))
		    total_bin++;
		if(pred_bin_labels[i].equals("Y") && act_bin_labels[i].equals("Y"))
		    corr_bin++;
	    }

	    String[] pred_labels = pred.label;
	    String[] act_labels = act.label;

	    HashMap corrMap = new HashMap();
	    for(int j = 0; j < act_labels.length; j++) {
		String line = act_labels[j];
		//if(line == null) continue;
		String[] toks = line.split("\t");
		if(toks.length > 2) {
		    Integer[] ids = new Integer[toks.length-1];
		    for(int i = 0; i < ids.length; i++)
			ids[i] = new Integer(Integer.parseInt(toks[i]));
		    Arrays.sort(ids);
		    line = "";
		    for(int i = 0; i < ids.length; i++)
			line += ids[i] + "\t";
		    line += toks[toks.length-1];
		    if(!corrMap.containsKey(line)) {
			corrMap.put(line,"");
			total++;
		    }
		}
	    }


	    HashMap alreadyMap = new HashMap();
	    for(int j = 0; j < pred_labels.length; j++) {
		String line = pred_labels[j];
		String[] toks = line.split("\t");
		Integer[] ids = new Integer[((toks.length-2)/2)];
		for(int i = 0; i < ids.length; i++)
		    ids[i] = new Integer(Integer.parseInt(toks[i]));
		Arrays.sort(ids);
		line = "";
		for(int i = 0; i < ids.length; i++)
		    line += ids[i] + "\t";
		line += toks[((toks.length-2)/2)];
		if(!alreadyMap.containsKey(line)) {
		    guessed++;
		    if(corrMap.containsKey(line))
			corr++;
		    alreadyMap.put(line,"");
		}
	    }
	}
	
	double prec = (double)corr_bin/guessed_bin;
	double rec = (double)corr_bin/total_bin;
	double fmeas = (2*prec*rec)/(prec+rec);
	System.out.println("BIN-CORR: " + corr_bin + ", BIN-GUESS: " + guessed_bin 
			   + ", BIN-ACT: " + total_bin);
	System.out.println("BIN-PREC: " + prec + ", BIN-REC: " + rec + ", BIN-FMEAS: " + fmeas);
	
	prec = (double)corr/guessed;
	rec = (double)corr/total;
	fmeas = (2*prec*rec)/(prec+rec);
	System.out.println("COM-CORR: " + corr + ", COM-GUESS: " + guessed + ", COM-ACT: " + total);
	System.out.println("COM-PREC: " + prec + ", COM-REC: " + rec + ", COM-FMEAS: " + fmeas);

	System.out.println("done.");
    }


}
