package struct.sequence;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import struct.types.Alphabet;
import cw_library.tools.MathTools;

public class CWSequencePredictor extends SequencePredictor {

	public CWSequencePredictor(int dimensions, double initSigmaValue,
			PredictCommitteeType committeeType) {
		super(dimensions);
		sigma = new double[dimensions];
		for (int i = 0; i < dimensions; i++) {
			sigma[i] = initSigmaValue;
		}
		setCommitteeType(committeeType);
	}

	public void saveModel(String file) throws Exception {
		System.out.print("Saving model ... ");
		ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(
				file));
		out.writeObject(weights);
		out.writeObject(sigma);
		out.writeObject(SequenceInstance.getDataAlphabet());
		out.writeObject(SequenceInstance.getTagAlphabet());
		out.close();
		System.out.println("done.");
	}

	public void loadModel(String file) throws Exception {
		System.out.print("Loading model ... ");
		ObjectInputStream in = new ObjectInputStream(new FileInputStream(file));
		weights = (double[]) in.readObject();
		sigma = (double[]) in.readObject();
		SequenceInstance.setDataAlphabet((Alphabet) in.readObject());
		SequenceInstance.setTagAlphabet((Alphabet) in.readObject());
		in.close();
		System.out.println("done.");
	}

	public double getMeanOfSigma() {
		return MathTools.calculateMean(sigma);
	}
}
