package struct.sequence;

import struct.sequence.SequencePredictor.PredictCommitteeType;
import struct.types.Features;
import struct.types.Instance;

public abstract class ConfidenceEstimator {

	public static ConfidenceEstimator resolveConfidenceEstimatorByName(
			String confEstimatorName) {

		String[] params = confEstimatorName.split("\\*");
		String type = params[0];

		if (type.equals("Random")) {
			return new ConfidenceEstimatorRandom();
		} else if (type.equals("Gamma")) {
			double coef = Double.parseDouble(params[1]);
			return new ConfidenceEstimatorGammaExp(coef);
		} else if (type.equals("Delta")) {
			return new ConfidenceEstimatorDelta();
		} else if (type.equals("KViter")) {
			int K = Integer.parseInt(params[1]);
			boolean weightedCommittee = false;
			return new ConfidenceEstimatorKViterbi(K, weightedCommittee);
		} else if (type.equals("WgtKViter")) {
			int K = Integer.parseInt(params[1]);
			boolean weightedCommittee = true;
			return new ConfidenceEstimatorKViterbi(K, weightedCommittee);
		} else if (type.equals("KDrawsByStddev")) {
			double multiplier = Double.parseDouble(params[1]);
			int K = Integer.parseInt(params[2]);
			return new ConfidenceEstimatorKDraws(multiplier, K,
					PredictCommitteeType.kDrawsByStdDev);
		} else if (type.equals("KDrawsByConst")) {
			double multiplier = Double.parseDouble(params[1]);
			int K = Integer.parseInt(params[2]);
			return new ConfidenceEstimatorKDraws(multiplier, K,
					PredictCommitteeType.kDrawsByConst);
		}

		throw new RuntimeException("Unknown confEstimatorName : "
				+ confEstimatorName);
	}

	public abstract void estimateInst(Instance inst, Features feats,
			LabelConfScore[] allWordsScores, SequencePredictor predictor);
}
