package struct.sequence;

import struct.sequence.SequencePredictor.PredictCommitteeType;
import struct.types.Features;
import struct.types.Instance;

public class ConfidenceEstimatorKViterbi extends ConfidenceEstimator {
	int k;
	boolean weightedCommittee;

	public ConfidenceEstimatorKViterbi(int k, boolean weightedCommittee) {
		this.k = k;
		this.weightedCommittee = weightedCommittee;
	}

	public void estimateInst(Instance inst, Features feats,
			LabelConfScore[] allWordsScores, SequencePredictor predictor) {
		// Use K-best Viterbi
		predictor.setCommitteeType(PredictCommitteeType.kBestViterbi);
		SequencePrediction kBestViterbiPrediction = predictor.decode(inst,
				feats, k);

		for (int wordIdx = 0; wordIdx < allWordsScores.length; wordIdx++) {
			double score;
			if (weightedCommittee) {
				score = ConfEstimation.evalWordConfByWeightedCommittee(
						kBestViterbiPrediction, k, wordIdx);
			} else {
				score = ConfEstimation.evalWordConfByCommittee(
						kBestViterbiPrediction, k, wordIdx);
			}
			allWordsScores[wordIdx].score = score;
		}
	}

}
