package struct.sequence;

import java.io.IOException;
import java.io.ObjectInputStream;

import struct.types.Alphabet;
import struct.types.FeatureVector;
import struct.types.Instance;

public class SequenceInstance implements Instance {

	protected SequenceInput input;
	protected SequenceLabel label;
	protected SequenceFeatures SequenceFeatures;

	protected static Alphabet tagAlphabet;
	protected static Alphabet dataAlphabet;

	public SequenceInstance() {
	}

	public SequenceInstance(String[] sentence, String[] pos, String[] tags,
			FeatureVector fv) {

		input = new SequenceInput(sentence, pos);

		label = new SequenceLabel(tags, fv, 0);

	}

	public SequenceInstance(String[] sentence, String[] tags, FeatureVector fv) {

		input = new SequenceInput(sentence);

		label = new SequenceLabel(tags, fv, 0);

	}

	public SequenceInput getInput() {
		return input;
	}

	public SequenceLabel getLabel() {
		return label;
	}

	public SequenceFeatures getFeatures() {
		return SequenceFeatures;
	}

	public SequenceFeatures getFeatures(ObjectInputStream in)
			throws IOException {

		String[] toks = input.sentence;

		FeatureVector fvs_ij[][][] = new FeatureVector[toks.length][tagAlphabet
				.size()][tagAlphabet.size()];
		FeatureVector fvs_j[][] = new FeatureVector[toks.length][tagAlphabet
				.size()];

		for (int k = 0; k < toks.length; k++) {
			for (int i = 0; i < tagAlphabet.size(); i++) {
				for (int j = 0; j < tagAlphabet.size(); j++) {
					FeatureVector prodFV = new FeatureVector(-1, -1.0, null);

					int indx = in.readInt();
					while (indx != -1) {
						prodFV = new FeatureVector(indx, 1.0, prodFV);
						indx = in.readInt();
					}

					fvs_ij[k][i][j] = prodFV;

				}
			}
		}

		int i1 = in.readInt();
		if (i1 != -2) {
			System.out.println("Feature Vector I/O problem 1:" + i1);
			System.out.println(toks[1] + "\t" + tagAlphabet.size());
			System.exit(0);
		}

		for (int k = 0; k < toks.length; k++) {
			for (int i = 0; i < tagAlphabet.size(); i++) {
				FeatureVector prodFV = new FeatureVector(-1, -1.0, null);

				int indx = in.readInt();
				while (indx != -1) {
					prodFV = new FeatureVector(indx, 1.0, prodFV);
					indx = in.readInt();
				}

				fvs_j[k][i] = prodFV;

			}
		}

		i1 = in.readInt();
		if (i1 != -3) {
			System.out.println("Feature Vector I/O problem 2:" + i1);
			System.out.println(toks[1] + "\t" + tagAlphabet.size() + "\t");
			System.exit(0);
		}

		return new SequenceFeatures(fvs_ij, fvs_j);

	}

	public static void setTagAlphabet(Alphabet tagAlphabet1) {
		tagAlphabet = tagAlphabet1;
	}

	public static Alphabet getTagAlphabet() {
		return tagAlphabet;
	}

	public static void setDataAlphabet(Alphabet dataAlphabet1) {
		dataAlphabet = dataAlphabet1;
	}

	public static Alphabet getDataAlphabet() {
		return dataAlphabet;
	}

	public void setFeatures(SequenceFeatures seqFeats) {
		SequenceFeatures = seqFeats;
	}

	public int tokensCount() {
		return input.sentence.length;
	}

}
