package struct.sequence;

import struct.types.FeatureVector;
import struct.types.Features;
import struct.types.Instance;

public class SequencePredictorByMajorityVote extends SequencePredictor {

	SequencePredictor predictor;
	int K;
	boolean O_ByMajority = false;
	boolean O_ForLowConfidence = false;
	boolean BI_ForLowConfidence = false;

	double lowConfidenceThreshold = 0;
	String confEstimatorName;

	public void setO_ByMajority(boolean oByMajority) {
		O_ByMajority = oByMajority;
	}

	public void setO_ForLowConfidence(boolean oForLowConfidence,
			double lowConfThreshold, String confEstimator) {
		O_ForLowConfidence = oForLowConfidence;
		lowConfidenceThreshold = lowConfThreshold;
		confEstimatorName = confEstimator;
	}

	public void setBI_ForLowConfidence(boolean bIForLowConfidence,
			double lowConfThreshold, String confEstimator) {
		BI_ForLowConfidence = bIForLowConfidence;
		lowConfidenceThreshold = lowConfThreshold;
		confEstimatorName = confEstimator;
	}

	public SequencePredictorByMajorityVote(SequencePredictor predictor,
			String predictByCommittee) {
		super(0);
		this.predictor = predictor;

		//
		// Expect: committeeType*Const*K
		//
		String[] vals = predictByCommittee.split("\\*");
		predictor.setCommitteeType(vals[0]);
		double drawsMultiplier = Double.parseDouble(vals[1]);
		predictor.setDrawsMultiplier(drawsMultiplier);
		K = Integer.parseInt(vals[2]);
	}

	public SequencePrediction decode(Instance inst, Features feats) {
		if (O_ByMajority) {
			return decode_O_ByMajority(inst, feats);
		}
		if (O_ForLowConfidence) {
			return decode_O_ForLowConfidence(inst, feats);
		}
		if (BI_ForLowConfidence) {
			return decode_BI_ForLowConfidence(inst, feats);
		}

		SequencePrediction predictions;
		predictions = predictor.decode(inst, feats, K);

		//
		// Collect votings
		//
		int tokensLen = predictions.getBestLabel().tags.length;
		int numOfTags = SequenceInstance.getTagAlphabet().size();
		int[][] votes = new int[tokensLen][numOfTags];
		SequenceLabel label;
		String[] labelTags;
		String tag;
		// Go over all K label sequences
		for (int rank = 0; rank < K; rank++) {
			label = predictions.getLabelByRank(rank);
			labelTags = label.tags;
			// go over all tags of current label
			for (int tokenIdx = 0; tokenIdx < tokensLen; tokenIdx++) {
				tag = labelTags[tokenIdx];
				int tagIdx = SequenceInstance.getTagAlphabet().lookupIndex(tag);
				// Inc by 1 the vote count of the current Tag for current Token
				votes[tokenIdx][tagIdx]++;
			}
		}

		//
		// Now produce majority-vote label sequence
		//
		String[] majorityVoteTags = new String[tokensLen];
		// Go voer all Tokens
		for (int tokenIdx = 0; tokenIdx < tokensLen; tokenIdx++) {
			// Find Tag with max votes
			int maxVotes = -1;
			int maxTagIdx = -1;
			for (int tagIdx = 0; tagIdx < numOfTags; tagIdx++) {
				if (votes[tokenIdx][tagIdx] > maxVotes) {
					maxVotes = votes[tokenIdx][tagIdx];
					maxTagIdx = tagIdx;
				}
			}
			String maxTag = (String) SequenceInstance.getTagAlphabet()
					.lookupObject(maxTagIdx);
			majorityVoteTags[tokenIdx] = maxTag;
		}

		FeatureVector fv = null;
		double prob = 0;
		SequenceLabel majorityVoteLabel = new SequenceLabel(majorityVoteTags,
				fv, prob);
		SequencePrediction majorityVotePrediction = new SequencePrediction(
				majorityVoteLabel);
		return majorityVotePrediction;

	}

	public SequencePrediction decode_O_ByMajority(Instance inst, Features feats) {
		SequencePrediction predictions;
		predictions = predictor.decode(inst, feats, K);

		//
		// Collect votings
		//
		int tokensLen = predictions.getBestLabel().tags.length;
		int numOfTags = SequenceInstance.getTagAlphabet().size();
		int[][] votes = new int[tokensLen][numOfTags];
		SequenceLabel label;
		String[] labelTags;
		String tag;
		// Go over all K label sequences
		for (int rank = 0; rank < K; rank++) {
			label = predictions.getLabelByRank(rank);
			labelTags = label.tags;
			// go over all tags of current label
			for (int tokenIdx = 0; tokenIdx < tokensLen; tokenIdx++) {
				tag = labelTags[tokenIdx];
				int tagIdx = SequenceInstance.getTagAlphabet().lookupIndex(tag);
				// Inc by 1 the vote count of the current Tag for current Token
				votes[tokenIdx][tagIdx]++;
			}
		}

		//
		// Now produce majority-vote label sequence
		//
		String[] majorityVoteTags = new String[tokensLen];
		// Go voer all Tokens
		for (int tokenIdx = 0; tokenIdx < tokensLen; tokenIdx++) {
			// Find Tag with max votes
			int maxVotes = -1;
			int maxTagIdx = -1;
			for (int tagIdx = 0; tagIdx < numOfTags; tagIdx++) {
				if (votes[tokenIdx][tagIdx] > maxVotes) {
					maxVotes = votes[tokenIdx][tagIdx];
					maxTagIdx = tagIdx;
				}
			}
			String maxTag = (String) SequenceInstance.getTagAlphabet()
					.lookupObject(maxTagIdx);

			if (maxTag.equals("O")) {
				majorityVoteTags[tokenIdx] = maxTag;
			} else {
				label = predictions.getBestLabel();
				labelTags = label.tags;
				majorityVoteTags[tokenIdx] = labelTags[tokenIdx];
			}
		}

		FeatureVector fv = null;
		double prob = 0;
		SequenceLabel majorityVoteLabel = new SequenceLabel(majorityVoteTags,
				fv, prob);
		SequencePrediction majorityVotePrediction = new SequencePrediction(
				majorityVoteLabel);
		return majorityVotePrediction;

	}

	public SequencePrediction decode_O_ForLowConfidence(Instance inst,
			Features feats) {
		SequencePrediction predictions;

		// Make a regular prediction
		predictions = predictor.decode(inst, feats);
		SequenceLabel label = predictions.getBestLabel();
		String[] labelTags = label.tags;
		int tokensLen = predictions.getBestLabel().tags.length;

		//
		// confidence estimation
		//
		ConfidenceEstimator confEstimator = ConfidenceEstimator
				.resolveConfidenceEstimatorByName(confEstimatorName);

		LabelConfScore[] allWordsScores = new LabelConfScore[tokensLen];
		int wordIdx;
		for (wordIdx = 0; wordIdx < allWordsScores.length; wordIdx++) {
			LabelConfScore wordScore = new LabelConfScore();
			allWordsScores[wordIdx] = wordScore;
		}

		//
		// Now assign the confidence score to the words
		//
		confEstimator.estimateInst(inst, feats, allWordsScores, predictor);

		//
		// Now produce new prediction
		//
		String[] correctedTags = new String[tokensLen];
		// Go voer all Tokens
		for (int tokenIdx = 0; tokenIdx < tokensLen; tokenIdx++) {
			if (allWordsScores[tokenIdx].score > lowConfidenceThreshold) {
				// High confidence tag, keep it
				correctedTags[tokenIdx] = labelTags[tokenIdx];
			} else {
				// low confidence label
				correctedTags[tokenIdx] = "O";
			}
		}

		FeatureVector fv = null;
		double prob = 0;
		SequenceLabel majorityVoteLabel = new SequenceLabel(correctedTags, fv,
				prob);
		SequencePrediction majorityVotePrediction = new SequencePrediction(
				majorityVoteLabel);
		return majorityVotePrediction;

	}

	public SequencePrediction decode_BI_ForLowConfidence(Instance inst,
			Features feats) {
		SequencePrediction predictions;

		predictions = predictor.decode(inst, feats, K);

		//
		// Collect votings
		//
		int tokensLen = predictions.getBestLabel().tags.length;
		int numOfTags = SequenceInstance.getTagAlphabet().size();
		int[][] votes = new int[tokensLen][numOfTags];
		SequenceLabel label;
		String[] labelTags;
		String tag;
		// Go over all K label sequences
		for (int rank = 0; rank < K; rank++) {
			label = predictions.getLabelByRank(rank);
			labelTags = label.tags;
			// go over all tags of current label
			for (int tokenIdx = 0; tokenIdx < tokensLen; tokenIdx++) {
				tag = labelTags[tokenIdx];
				int tagIdx = SequenceInstance.getTagAlphabet().lookupIndex(tag);
				// Inc by 1 the vote count of the current Tag for current Token
				votes[tokenIdx][tagIdx]++;
			}
		}

		//
		// confidence estimation
		//
		ConfidenceEstimator confEstimator = ConfidenceEstimator
				.resolveConfidenceEstimatorByName(confEstimatorName);

		LabelConfScore[] allWordsScores = new LabelConfScore[tokensLen];
		int wordIdx;
		for (wordIdx = 0; wordIdx < allWordsScores.length; wordIdx++) {
			LabelConfScore wordScore = new LabelConfScore();
			allWordsScores[wordIdx] = wordScore;
		}

		//
		// Now assign the confidence score to the words
		//
		confEstimator.estimateInst(inst, feats, allWordsScores, predictor);

		//
		// Now produce new prediction
		//
		String[] correctedTags = new String[tokensLen];
		labelTags = predictions.getBestLabel().tags;
		// Go over all Tokens
		for (int tokenIdx = 0; tokenIdx < tokensLen; tokenIdx++) {
			if (allWordsScores[tokenIdx].score > lowConfidenceThreshold) {
				// High confidence tag, keep it
				correctedTags[tokenIdx] = labelTags[tokenIdx];
			} else {
				// low confidence label
				if (labelTags[tokenIdx].equals("O")) {
					// replace low confidence "O" by best scoring B\I
					// Find Tag with max votes
					int maxVotes = -1;
					int maxTagIdx = -1;
					int secondMaxVotes = -1;
					int secondMaxTagIdx = -1;
					for (int tagIdx = 0; tagIdx < numOfTags; tagIdx++) {
						if (votes[tokenIdx][tagIdx] > maxVotes) {
							secondMaxVotes = maxVotes;
							secondMaxTagIdx = maxTagIdx;
							maxVotes = votes[tokenIdx][tagIdx];
							maxTagIdx = tagIdx;
						} else if (votes[tokenIdx][tagIdx] > secondMaxVotes) {
							secondMaxVotes = votes[tokenIdx][tagIdx];
							secondMaxTagIdx = tagIdx;
						}
					}
					String maxTag = (String) SequenceInstance.getTagAlphabet()
							.lookupObject(maxTagIdx);
					String secondMaxTag = (String) SequenceInstance
							.getTagAlphabet().lookupObject(secondMaxTagIdx);

					if (maxTag.equals("O")) {
						correctedTags[tokenIdx] = secondMaxTag;
					} else {
						correctedTags[tokenIdx] = maxTag;
					}

				} else {
					// Any B\I keep as is
					correctedTags[tokenIdx] = labelTags[tokenIdx];
				}
			}
		}

		FeatureVector fv = null;
		double prob = 0;
		SequenceLabel majorityVoteLabel = new SequenceLabel(correctedTags, fv,
				prob);
		SequencePrediction majorityVotePrediction = new SequencePrediction(
				majorityVoteLabel);
		return majorityVotePrediction;

	}
}
