package struct.sequence;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

import struct.types.Alphabet;
import struct.types.FeatureVector;
import struct.types.Features;
import struct.types.Instance;

public class SequenceStatistics {

	public void stat(Instance[] training, String featureFile,
			Alphabet tagAlphabet) throws IOException {

		long start = System.currentTimeMillis();
		int numLabels = tagAlphabet.size();
		ObjectInputStream in = new ObjectInputStream(new FileInputStream(
				featureFile));

		double avg_vertex = 0;
		double avg_vertex2 = 0;
		double avg_edge = 0;
		double avg_edge2 = 0;
		long n_vertex = 0;
		long n_edge = 0;
		double max_vertex = Double.NEGATIVE_INFINITY;
		double max_edge = Double.NEGATIVE_INFINITY;

		FeatureVector fv;
		double nrm;

		for (int j = 0; j < training.length; j++) {

			Instance inst = training[j];
			Features feats = training[j].getFeatures(in);

			SequenceInstance sequence_inst = (SequenceInstance) inst;
			SequenceLabel sequence_label = sequence_inst.getLabel();
			String[] actual_tags = sequence_label.tags;
			int numTokens = actual_tags.length; // length of sequence
			SequenceFeatures sequence_feats = (SequenceFeatures) feats;

			for (int t = 0; t < numTokens; t++) {
				for (int l = 0; l < numLabels; l++) {
					n_vertex++;
					fv = sequence_feats.getFeatureVector(t, l);
					nrm = FeatureVector.twoNorm(fv);
					avg_vertex += nrm;
					avg_vertex2 += (nrm * nrm);
					if (nrm > max_vertex)
						max_vertex = nrm;

				}
			}

			for (int t = 0; t < numTokens; t++) {
				for (int l1 = 0; l1 < numLabels; l1++) {
					for (int l2 = 0; l2 < numLabels; l2++) {
						n_edge++;
						fv = sequence_feats.getFeatureVector(t, l1, l2);
						nrm = FeatureVector.twoNorm(fv);
						avg_edge += nrm;
						avg_edge2 += (nrm * nrm);
						if (nrm > max_edge)
							max_edge = nrm;

					}
				}

			}
		}

		System.out.println("n_vertex   = " + n_vertex);
		System.out.println("avg_vertex = " + avg_vertex / n_vertex);
		System.out.println("sqrt(avg_vertex2) = "
				+ Math.sqrt(avg_vertex2 / n_vertex));
		System.out.println("max_vertex = " + max_vertex);

		System.out.println("n_edge   = " + n_edge);
		System.out.println("avg_edge = " + avg_edge / n_edge);
		System.out
				.println("sqrt(avg_edge2) = " + Math.sqrt(avg_edge2 / n_edge));
		System.out.println("max_edge = " + max_edge);

		in.close();

		long end = System.currentTimeMillis();
		System.out.println("Statistics took: " + (end - start));
	}

}
