/* Copyright (C) 2002 Univ. of Massachusetts Amherst, Computer Science Dept.
   This file is part of "MALLET" (MAchine Learning for LanguagE Toolkit).
   http://www.cs.umass.edu/~mccallum/mallet
   This software is provided under the terms of the Common Public License,
   version 1.0, as published by http://www.opensource.org.  For further
   information, see the file `LICENSE' included with this distribution. */


/**
   @author Ryan McDonald <a href="mailto:ryantm@cis.upenn.edu">ryantm@cis.upenn.edu</a>
 */

import edu.umass.cs.mallet.base.pipe.*;
import edu.umass.cs.mallet.base.types.*;

public class HW2TokenSequence extends Pipe
{
	int degree;
	double tran_val,node_val;
	public HW2TokenSequence (int degree)
	{
		super (null, LabelAlphabet.class);
		this.degree = degree;
		if(degree == 2) {
			tran_val = 29.0; node_val = Math.sqrt(2);
		}
		else {
			tran_val = 5.29; node_val = 1.0;
		}
	}

	public Instance pipe (Instance carrier)
	{
		String sentenceLines = (String) carrier.getData();
		String[] tokens = sentenceLines.split ("\n");
		TokenSequence data = new TokenSequence (tokens.length);
		LabelSequence target = new LabelSequence ((LabelAlphabet)getTargetAlphabet(),
							  tokens.length);

		String label;
		String[] feats;
		for (int i = 0; i < tokens.length; i++) {
			String[] features = tokens[i].split (" ");
			label = features[0];
			feats = new String[features.length-1];
			for(int j = 0; j < feats.length; j++)
				feats[j] = features[j+1];


			Token token = new Token (tokens[i]);
			for(int j = 0; j < feats.length; j++) {
				token.setFeatureValue(feats[j],1.0);
				for(int k = j+1; k < feats.length && degree > 1; k++)
					token.setFeatureValue(feats[j]+"_"+feats[k],node_val);
			}
			
			// Append
			data.add (token);
			target.add (label);
			
		}
		//System.out.println ("");
		carrier.setData(data);
		carrier.setTarget(target);
		return carrier;
	}
}
