/* Copyright (C) 2002 Univ. of Massachusetts Amherst, Computer Science Dept.
   This file is part of "MALLET" (MAchine Learning for LanguagE Toolkit).
   http://www.cs.umass.edu/~mccallum/mallet
   This software is provided under the terms of the Common Public License,
   version 1.0, as published by http://www.opensource.org.  For further
   information, see the file `LICENSE' included with this distribution. */




/** 
   @author Ryan McDonald <a href="mailto:ryantm@cis.upenn.edu">ryantm@cis.upenn.edu</a>
 */

//package edu.umass.cs.mallet.users.ryantm.medline;

import edu.umass.cs.mallet.base.types.*;
import edu.umass.cs.mallet.base.fst.*;
import edu.umass.cs.mallet.base.pipe.*;
import edu.umass.cs.mallet.base.pipe.iterator.*;
import edu.umass.cs.mallet.base.pipe.tsf.*;
import junit.framework.*;
import java.util.Iterator;
import java.util.Random;
import java.util.regex.*;
import java.io.*;

public class hw
{
	
	public static void main (String[] args) throws FileNotFoundException, Exception
	{

		CRF4 crf = null;
		InstanceList trainingData, testingData = null;

		int degree = 2;
		
		Pipe p = new SerialPipes (new Pipe[] {
			new HW2TokenSequence (degree),
			new TokenSequence2FeatureVectorSequence (false, true)
		});
		
		trainingData = new InstanceList (p);
		trainingData.add (new LineGroupIterator						
				  (new FileReader (new File (args[0])), Pattern.compile("^$"), true));
		
		testingData = new InstanceList (p);
		testingData.add (new LineGroupIterator
				 (new FileReader (new File (args[1])), Pattern.compile("^$"), true));

		Alphabet dataAlpha = p.getDataAlphabet();
		augmentAlpha(dataAlpha,degree);
		
		System.out.println ("Number of predicates in total: "+p.getDataAlphabet().size());
		p.getDataAlphabet().stopGrowth();

		crf = null;
		TokenAccuracyEvaluator eval =
			new TokenAccuracyEvaluator ();
		
		crf = new CRF4 (p, null);
		crf.setUseSparseWeights (false);
		crf.addOrderNStates(trainingData, new int[] {0,1},
				    new boolean[] {false,true},"a",null,null,true);
		crf.setGaussianPriorVariance (1.0);
		
		System.out.println("Training on "+trainingData.size()+" training instances, "+
				   testingData.size()+" testing instances...");
		
		crf.train (trainingData, null, null, null, 30);
		eval.test(crf,trainingData,"Training Performance:",null);
		eval.test(crf,testingData,"Testing Performance:",null);
	}

	private static void augmentAlpha(Alphabet dataAlpha, int degree) {
		for(int i = 0; i < 128; i++) {
			dataAlpha.lookupIndex(i+"");
			for(int j = i+1; j < 128 && degree == 2; j++)
				dataAlpha.lookupIndex(i+"_"+j);
		}
	}

}
