package struct.alg;

import struct.types.*;

import java.io.*;

public class OnlineLearner implements StructuredLearner {

    public void train(Instance[] training, OnlineUpdator update,
		      Predictor predictor, int numIters, String featureFile) throws IOException {

	for(int i = 0; i < numIters; i++) {

	    System.out.println("==========================");
	    System.out.println("Training iteration: " + i);
	    System.out.println("==========================");
	    long start = System.currentTimeMillis();

	    ObjectInputStream in = new ObjectInputStream(new FileInputStream(featureFile));

	    for(int j = 0; j < training.length; j++) {
		System.out.print(".");

		Instance inst = training[j];
		Features feats = training[j].getFeatures(in);

		double avg_upd = (double)(numIters*training.length - (training.length*((i+1)-1)+(j+1)) + 1);

		update.update(inst,feats,predictor,avg_upd);

	    }

	    System.out.println("");

	    in.close();

	    if(true) {
		System.out.println("Averaging parameters...");
		predictor.averageWeights(numIters*training.length);
	    }

	    long end = System.currentTimeMillis();
	    System.out.println("Training took: " + (end-start));

	}

    }

    public void trainAndEvaluate(Instance[] training,
				 Instance[] testing,
				 OnlineUpdator update,
				 Predictor predictor,
				 int numIters,
				 String trainingFeatureFile,
				 String testingFeatureFile,
				 Evaluator eval) throws IOException {

	trainAndEvaluate(training,testing,update,predictor,numIters,
			 trainingFeatureFile,testingFeatureFile,eval,true);

    }

    public void trainAndEvaluate(Instance[] training,
				 Instance[] testing,
				 OnlineUpdator update,
				 Predictor predictor,
				 int numIters,
				 String trainingFeatureFile,
				 String testingFeatureFile,
				 Evaluator eval,
				 boolean avgParams) throws IOException {

	

	for(int i = 0; i < numIters; i++) {
	    //System.out.print(i + " ");
	    System.out.println("==========================");
	    System.out.println("Training iteration: " + i);
	    System.out.println("==========================");
	    long start = System.currentTimeMillis();

	    ObjectInputStream in = new ObjectInputStream(new FileInputStream(trainingFeatureFile));

	    for(int j = 0; j < training.length; j++) {
		//System.out.print(".");

		Instance inst = training[j];
		Features feats = training[j].getFeatures(in);

		double avg_upd = (double)(numIters*training.length - (training.length*((i+1)-1)+(j+1)) + 1);

		update.update(inst,feats,predictor,avg_upd);

	    }

	    //System.out.println("");

	    in.close();

	    long end = System.currentTimeMillis();
	    System.out.println("Training took: " + (end-start));
	    System.out.println("Training");
	    eval.evaluate(training,predictor,trainingFeatureFile);
	    System.out.println("Testing");
	    eval.evaluate(testing,predictor,testingFeatureFile);
	}
	System.out.println("");

	if(avgParams) {
	    System.out.println("Averaging parameters...");
	    predictor.averageWeights(numIters*training.length);
	}
	System.out.println("==========================");
	System.out.println("Final Performance.");
	System.out.println("==========================");
	System.out.println("Training");
	eval.evaluate(training,predictor,trainingFeatureFile);
	System.out.println("Testing");
	eval.evaluate(testing,predictor,testingFeatureFile);

    }



}
