package struct.discontinuous;

import struct.types.*;
import struct.sequence.*;

import java.io.*;
import gnu.trove.*;
import java.util.*;

public class ContinuousDataManager extends SequenceDataManager {

    public String fileExt;

    public ContinuousDataManager()  {
	super();
	tagAlphabet.lookupIndex("O");
	tagAlphabet.lookupIndex("B-S");
	tagAlphabet.lookupIndex("I-S");
	fileExt = ".feats";
    }

    public SequenceInstance[] readData(String file) throws IOException {
	return readData(file,true);
    }

    public String[][] getLines(BufferedReader in) throws IOException {
	ArrayList lines = new ArrayList();
	String line = in.readLine();
	while(line != null && line.trim().length() > 0) {
	    lines.add(line.split("\t"));
	    line = in.readLine();
	}
	String[][] result = new String[lines.size()][];
	for(int i = 0; i < result.length; i++)
	    result[i] = (String[])lines.get(i);
	return result;
    }
    
    public SequenceInstance[] readData(String file, boolean createFeatureFile) throws IOException {
	
	BufferedReader in =
	    new BufferedReader(new InputStreamReader(new FileInputStream(file),"8859_2"));
	String[][] lines = getLines(in);
	
	LinkedList lt = new LinkedList();
	
	ObjectOutputStream out = 
	    createFeatureFile ? new ObjectOutputStream(new FileOutputStream(file+fileExt)) : null;
	
	int num1 = 0;
	System.out.println("Creating feature vectors and/or forests ...");
	while(lines.length > 0) {
	    if(num1 > 0 && num1 % 500 == 0)
		System.out.println("Creating Feature Vector Instance: " + num1
				   + ", Num Feats: " + dataAlphabet.size());
	    
	    String[] toks = new String[lines[0].length+1];
	    toks[0] = "<init>";
	    for(int i = 1; i < toks.length; i++)
		toks[i] = lines[0][i-1];
	    String[] labels = new String[lines[0].length+1];
	    for(int i = 0; i < labels.length; i++)
		labels[i] = "O";
	    for(int i = 0; i < lines.length; i++) {
		String res = "";
		for(int j = 1; j < lines[i].length; j++)
		    res += lines[i][j];
		if(!res.matches(".*I[O]+I.*")) { // If this is continuous
		    for(int j = 0; j < lines[i].length; j++) {
			if(lines[i][j].equals("I"))
			    labels[j+1] = "I-S";
		    }
		}
	    }
	    for(int i = 1; i < labels.length; i++)
		if(labels[i].equals("I-S") && labels[i-1].equals("O"))
		    labels[i] = "B-S";
	    String[][] orig_labels = new String[lines.length-1][lines[0].length+1];
	    for(int i = 0; i < orig_labels.length; i++) {
		orig_labels[i][0] = "I";
		for(int j = 1; j < orig_labels[i].length; j++)
		    orig_labels[i][j] = lines[i+1][j-1];
	    }
	    
	    LinkedList[] predicates = getPredicates(toks,null);
	    FeatureVector fv = 
		createFeatureVector(predicates,labels,new FeatureVector(-1,-1.0,null));
		    
	    ContinuousInstance si = new ContinuousInstance(toks,labels,fv);
	    si.all_labels = orig_labels;

	    if(createFeatureFile)
		createForest(si,predicates,out);
	    
	    lt.add(si);
	    
	    lines = getLines(in);
			
	    num1++;
	}
		
	SequenceInstance[] si = new SequenceInstance[lt.size()];
	for(int i = 0; i < si.length; i++) {
	    si[i] = (SequenceInstance)lt.get(i);
	}

	if(createFeatureFile)
	    out.close();
		
	return si;
		
    }

    public void createAlphabets(String file) throws IOException {
	createAlphabets(file,false);
    }

    public void createAlphabets(String file, boolean createUnsupported) throws IOException {
	System.out.print("Creating Data Alphabet ... ");

	    
	BufferedReader in =
	    new BufferedReader(new InputStreamReader(new FileInputStream(file),"8859_2"));
	String[][] lines = getLines(in);
		
	while(lines.length > 0) {
	    
	    String[] toks = new String[lines[0].length+1];
	    toks[0] = "<init>";
	    for(int i = 1; i < toks.length; i++)
		toks[i] = lines[0][i-1];
	    String[] labels = new String[lines[0].length+1];
	    for(int i = 0; i < labels.length; i++)
		labels[i] = "O";
	    for(int i = 0; i < lines.length; i++) {
		String res = "";
		for(int j = 1; j < lines[i].length; j++)
		    res += lines[i][j];
		if(!res.matches(".*I[O]+I.*")) { // If this is continuous
		    for(int j = 0; j < lines[i].length; j++) {
			if(lines[i][j].equals("I"))
			    labels[j+1] = "I-S";
		    }
		}
	    }
	    for(int i = 1; i < labels.length; i++)
		if(labels[i].equals("I-S") && labels[i-1].equals("O"))
		    labels[i] = "B-S";
	    
	    LinkedList[] predicates = getPredicates(toks,null);
	    FeatureVector fv = 
		createFeatureVector(predicates,labels,new FeatureVector(-1,-1.0,null));
		    
	    SequenceInstance si = new SequenceInstance(toks,labels,fv);

	    if(createUnsupported)
		createU(predicates);
	    
	    lines = getLines(in);
			
	}
	
	System.out.println("done.");
    }
	

    public LinkedList[] getPredicates(String[] toks, String[] pos) {
	String CAPS = "[A-Z]";
	String LOW = "[a-z]";
	String CAPSNUM = "[A-Z0-9]";
	String ALPHA = "[A-Za-z]";
	String ALPHANUM = "[A-Za-z0-9]";
	String PUNT = "[,\\.;:?!]";

	LinkedList[] predicates = new LinkedList[toks.length];
		
	for(int i = 0; i < toks.length; i++) {
	    predicates[i] = new LinkedList();

	    String word = toks[i];

	    // ortho predicates
	    if(word.matches(CAPS+".*"))
		predicates[i].add("ORT=INITCAP");
	    if(word.matches(CAPS+LOW+"*"))
		predicates[i].add("ORT=CAPS");
	    if(word.matches(CAPS+"+"))
		predicates[i].add("ORT=ALLCAPS");
	    if(word.matches("[ivxdlcm]+|[IVXDLCM]+"))
		predicates[i].add("ORT=ROMAN");
	    if(word.matches("\\.\\.+"))
		predicates[i].add("ORT=MULTIDOT");
	    if(word.matches("[^\\.]+.*\\."))
		predicates[i].add("ORT=ENDDOT");
	    if(word.matches(ALPHANUM+"+-"+ALPHANUM+"*"))
		predicates[i].add("ORT=DASH");
	    if(word.matches("[A-Z][A-Z\\.]*\\.[A-Z\\.]*"))
		predicates[i].add("ORT=ACRO");
	    if(word.matches(CAPS+"\\."))
		predicates[i].add("ORT=INIT");
	    if(word.matches(ALPHA))
		predicates[i].add("ORT=SING");
	    if(word.matches(CAPS))
		predicates[i].add("ORT=CAP1");
	    if(word.matches(PUNT))
		predicates[i].add("ORT=PUNC");
			

	    String[] toksL = new String[toks.length];
	    for(int j = 0; j < toks.length; j++)
		toksL[j] = toks[j].toLowerCase();
	    
	    // word and suffix features
	    predicates[i].add("WORD="+toksL[i]);
	    if(i > 0) {
		predicates[i].add("WORD-1="+toksL[i-1]);
		predicates[i].add("WORD="+toksL[i] + "_WORD-1="+toksL[i-1]);
	    }
	    if(i < toks.length-1) {
		predicates[i].add("WORD+1="+toksL[i+1]);
		predicates[i].add("WORD="+toksL[i]+"_WORD+1="+toksL[i+1]);
	    }
	    for(int j = 2; j < 5; j++) {
		if(toksL[i].length() > j) {
		    predicates[i].add("PREF"+j+"="+toksL[i].substring(0,j));
		    predicates[i].add("SUFF"+j+"="+
				      toksL[i].substring(toksL[i].length()-j,
							 toksL[i].length()));
		}
	    }
	}
		
	return predicates;
		
    }
}
