package struct.discontinuous;

import struct.sequence.*;
import struct.types.*;
import struct.alg.*;

import gnu.trove.*;

import java.io.*;
import java.text.DecimalFormat;
import java.util.HashMap;

public class ContinuousEvaluator implements Evaluator {


    public void evaluate(Instance[] data, Predictor pred) {
	System.out.println("Not implemented.");
    }

    public void evaluate(Instance[] data, Predictor predictor, String featureFile) throws IOException {
	
	int guessed = 0;
	int real = 0;
	int correct = 0;

	int corr_prec = 0;
	
	System.out.println("Evaluating ... ");

	ObjectInputStream in = new ObjectInputStream(new FileInputStream(featureFile));

	for(int k = 0; k < data.length; k++) {

	    Instance inst = data[k];
	    Features feats = data[k].getFeatures(in);

	    SequencePrediction prediction = ((SequencePredictor)predictor).decode(inst,feats);

	    String[][] act_labels = ((ContinuousInstance)inst).all_labels;
	    String[][] pred_labels = getPredLabels(((SequenceLabel)prediction.getBestLabel()).tags);

	    HashMap act = getMap(act_labels);
	    HashMap pred = getMap(pred_labels);

	    guessed += pred.size();
	    real += act.size();

	    Object[] keys = pred.keySet().toArray();
	    for(int i = 0; i < keys.length; i++) {
		String p = (String)keys[i];
		if(act.containsKey(keys[i])) {
		    correct++;
		}
	    }
	}

	in.close();
	DecimalFormat nf = new DecimalFormat();
	nf.setMaximumFractionDigits(5);
 
	System.out.println("Total:");
	System.out.println("\tGuessed: " + guessed + ", Actual: " 
			   + real + ", Correct: " + correct);
	double prec = (double)correct/guessed;
	double rec = (double)correct/real;
	double fmeas = (2*prec*rec)/(prec+rec);
	System.out.println("\tPrecision: " + nf.format(prec)
			   + ", Recall: " + nf.format(rec) 
			   + ", F-meas: " + nf.format(fmeas));

	System.out.println("done.");
    }

    private String[][] getPredLabels(String[] labels) {
	int numE = 0;
	for(int i = 0; i < labels.length; i++)
	    if(labels[i].startsWith("B"))
		numE++;
	String[][] pred_labels = new String[numE][labels.length];
	for(int i = 0; i < numE; i++) {
	    pred_labels[i][0] = "I";
	    for(int j = 1; j < labels.length; j++)
		pred_labels[i][j] = "O";
	}

	int wh = -1;
	for(int i = 0; i < labels.length; i++) {
	    if(labels[i].startsWith("B")) {
		wh++;
		pred_labels[wh][i] = "I";
		int j = i+1;
		while(j < labels.length && labels[j].startsWith("I")) {
		    pred_labels[wh][j] = "I";
		    j++;
		}
	    }
	}
	return pred_labels;
    }

    public HashMap getMap(String[][] labels) {
	HashMap labMap = new HashMap();
	for(int i = 0; i < labels.length; i++) {
	    String res = "";
	    for(int j = 0; j < labels[i].length; j++)
		res += labels[i][j] + " ";
	    labMap.put(res.trim(),"");
	}
	return labMap;
    }

 }
