package struct.discontinuous;

import struct.types.*;

import java.io.*;
import gnu.trove.*;
import java.util.*;

public class DiscontinuousDataManager implements DataManager {

    public Alphabet dataAlphabet;
    public boolean usePOS;
 
    public DiscontinuousDataManager()  {
	dataAlphabet = new Alphabet();
	usePOS = false;
    }

    public void setUsePOS(boolean usePOS) { this.usePOS = usePOS; }

    public Alphabet getDataAlphabet() { return dataAlphabet; }

    public DiscontinuousInstance[] readData(String file) throws IOException {
	return readData(file,true);
    }

    public String[][] getLines(BufferedReader in) throws IOException {
	ArrayList lines = new ArrayList();
	String line = in.readLine();
	while(line != null && line.trim().length() > 0) {
	    lines.add(line.split("\t"));
	    line = in.readLine();
	}
	String[][] result = new String[lines.size()][];
	for(int i = 0; i < result.length; i++)
	    result[i] = (String[])lines.get(i);
	return result;
    }
    
    public DiscontinuousInstance[] readData(String file, boolean createFeatureFile) throws IOException {
	
	BufferedReader in =
	    new BufferedReader(new InputStreamReader(new FileInputStream(file),"8859_2"));
	String[][] lines = getLines(in);
	
	LinkedList lt = new LinkedList();
	
	ObjectOutputStream out = 
	    createFeatureFile ? new ObjectOutputStream(new FileOutputStream(file+".feats")) : null;
	
	int num1 = 0;
	System.out.println("Creating feature vectors and/or forests ...");
	while(lines.length > 0) {
	    if(num1 > 0 && num1 % 500 == 0)
		System.out.println("Creating Feature Vector Instance: " + num1
				   + ", Num Feats: " + dataAlphabet.size());
	    
	    String[] toks = new String[lines[0].length+2];
	    String[] pos = null;
	    toks[0] = "<init>";
	    toks[toks.length-1] = "<ending>";
	    for(int i = 1; i < toks.length-1; i++)
		toks[i] = lines[0][i-1];
	    if(usePOS) {
		pos = new String[lines[0].length+2];
		pos[0] = "<initPOS>";
		pos[toks.length-1] = "<endingPOS>";
		for(int i = 1; i < pos.length-1; i++)
		    pos[i] = lines[1][i-1];
	    }
	    int numLabs = usePOS ? lines.length-2 : lines.length-1;
	    String[][] labels = new String[numLabs][lines[0].length+2];
	    for(int i = 0; i < labels.length; i++) {
		labels[i][0] = "I";
		labels[i][labels[i].length-1] = "I";
		for(int j = 1; j < labels[i].length-1; j++)
		    labels[i][j] = lines[i+(usePOS ? 2 : 1)][j-1];
	    }
	    
	    FeatureVector[] fv = new FeatureVector[labels.length];
	    for(int i = 0; i < fv.length; i++) {
		fv[i] = createFeatureVector(toks,pos,labels[i],new FeatureVector(-1,-1.0,null));
		fv[i].sort();
	    }
	    
	    DiscontinuousInstance si = new DiscontinuousInstance(toks,pos,labels,fv);
	    
	    if(createFeatureFile)
		createForest(si,out);
	    
	    lt.add(si);
	    
	    lines = getLines(in);
			
	    num1++;
	}
		
	DiscontinuousInstance[] si = new DiscontinuousInstance[lt.size()];
	for(int i = 0; i < si.length; i++) {
	    si[i] = (DiscontinuousInstance)lt.get(i);
	}

	if(createFeatureFile)
	    out.close();
		
	return si;
		
    }

    public void createAlphabets(String file) throws IOException {
	createAlphabets(file,false);
    }

    public void createAlphabets(String file, boolean createUnsupported) throws IOException {
	System.out.print("Creating Data Alphabet ... ");

	BufferedReader in =
	    new BufferedReader(new InputStreamReader(new FileInputStream(file),"8859_2"));
	String[][] lines = getLines(in);
				
	int num1 = 0;
	while(lines.length > 0) {

	    String[] toks = new String[lines[0].length+2];
	    String[] pos = null;
	    toks[0] = "<init>";
	    toks[toks.length-1] = "<ending>";
	    for(int i = 1; i < toks.length-1; i++)
		toks[i] = lines[0][i-1];
	    if(usePOS) {
		pos = new String[lines[0].length+2];
		pos[0] = "<initPOS>";
		pos[toks.length-1] = "<endingPOS>";
		for(int i = 1; i < pos.length-1; i++)
		    pos[i] = lines[1][i-1];
	    }
	    int numLabs = usePOS ? lines.length-2 : lines.length-1;
	    String[][] labels = new String[numLabs][lines[0].length+2];
	    for(int i = 0; i < labels.length; i++) {
		labels[i][0] = "I";
		labels[i][labels[i].length-1] = "I";
		for(int j = 1; j < labels[i].length-1; j++)
		    labels[i][j] = lines[i+(usePOS ? 2 : 1)][j-1];
	    }

	    FeatureVector[] fv = new FeatureVector[labels.length];
	    for(int i = 0; i < fv.length; i++) {
		fv[i] = createFeatureVector(toks,pos,labels[i],new FeatureVector(-1,-1.0,null));
		fv[i].sort();
	    }
	    
	    DiscontinuousInstance si = new DiscontinuousInstance(toks,pos,labels,fv);
	    
	    if(createUnsupported) {
		for(int i = 0; i < toks.length; i++) {
		    for(int j = i+1; j < toks.length; j++) {
			FeatureVector nfv = 
			    createFeatureVector(toks,pos,i,j,new FeatureVector(-1,-1.0,null));
		    }
		}
	    }
	    
	    lines = getLines(in);
			
	    num1++;
	}
	
	System.out.println("done.");
    }
	

    private FeatureVector createFeatureVector(String[] toks,
					      int n1,
					      int n2, FeatureVector fv) {
	return createFeatureVector(toks,null,n1,n2,fv);
    }

    private FeatureVector createFeatureVector(String[] toks, String[] pos,
					      int n1,
					      int n2, FeatureVector fv) {

	// No features on for prev=strt
	//if(n1 == 0) return fv;

	int dist = n2-n1;
	String dt = "11";

	if(dist <= 10) dt = "10";
	if(dist <= 8) dt = "8";
	if(dist <= 5) dt = "5";
	if(dist <= 3) dt = "3";
	if(dist <= 2) dt = "2";
	if(dist < 1) dt = "1";

	if(n1 != 0 && n2 != toks.length-1) {
	    fv = fv.add("DIST="+dt,1.0,dataAlphabet);
	    fv = fv.add("DISTA="+dist,1.0,dataAlphabet);
	}

	String[] toksL = new String[toks.length];
	for(int i = 0; i < toks.length; i++)
	    toksL[i] = toks[i].toLowerCase();

	for(int k = 0; k < 2; k++) {
	    String end = k == 0 ? "" : "_D="+dt;

	    if((n1 == 0 || n2 == toks.length-1) && k ==1) break;

	    // add other conj?
	    fv = fv.add("PWORD="+toksL[n1]+"_TWORD="+toksL[n2]+end,1.0,dataAlphabet);
	    for(int i = n1+1; i < n2 && n1 != 0 && n2 != toks.length-1; i++) {
		fv = fv.add("INB"+toksL[i]+end,1.0,dataAlphabet);
		if(toksL[i].matches("[,\\.;:\\-/]"))
		    fv = fv.add("INBPUNC"+end,1.0,dataAlphabet);
		if(toksL[i].matches("[0-9]+"))
		   fv = fv.add("INBNUM"+end,1.0,dataAlphabet);
	    }
	}

	// Create node features
	fv = createFeatureVector(toks,pos,n2,"I",fv);

	if(n2 == toks.length-1) {
	    fv = createFeatureVector(toks,pos,n1,"ENDAA",fv);
	    fv = createFeatureVector(toks,pos,n1+1,"END",fv); // REAL ONE
	}
	if(n1 == 0) {
	    fv = createFeatureVector(toks,pos,n2,"START",fv); // REAL ONE
	    fv = createFeatureVector(toks,pos,n2-1,"STARTAA",fv);
	}

	// If we are just adding another
	if(n1 == n2-1 && n1 != 0 && n2 != toks.length-1)
	    fv = createFeatureVector(toks,pos,n2,"I&I",fv);
	else if(n1 != 0 && n2 != toks.length-1 && n1 != n2-1) {
	    fv = createFeatureVector(toks,pos,n1,"B-DIS",fv);
	    fv = createFeatureVector(toks,pos,n2,"E-DIS",fv);
	}

	return fv;
    }
	
    private FeatureVector createFeatureVector(String[] toks, String[] pos,
					      int n, String label,
					      FeatureVector fv) {

	if(n == 0 || n == toks.length-1)
	    return fv;

	String[] toksL = new String[toks.length];
	for(int i = 0; i < toks.length; i++)
	    toksL[i] = toks[i].toLowerCase();

	fv = orthoFeatures(toks[n],label,fv);

	// word and suffix features
	fv = fv.add("WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
	fv = fv.add("WORD-1="+toksL[n-1]+"_"+label,1.0,dataAlphabet);
	fv = fv.add("WORD+1="+toksL[n+1]+"_"+label,1.0,dataAlphabet);
	fv = fv.add("WORD="+toksL[n]+"_WORD-1="+toksL[n-1]+"_"+label,1.0,dataAlphabet);
	fv = fv.add("WORD="+toksL[n]+"_WORD+1="+toksL[n+1]+"_"+label,1.0,dataAlphabet);
	for(int i = 2; i < 5; i++) {
	    if(toksL[n].length() > i) {
		fv = fv.add("PREF"+i+"="+toksL[n].substring(0,i)+"_"+label,1.0,dataAlphabet);
		fv = fv.add("SUFF"+i+"="+
			    toksL[n].substring(toksL[n].length()-i,toksL[n].length())+"_"+label,
			    1.0,dataAlphabet);
	    }
	}

	if(pos != null) {
	    // pos features
	    fv = fv.add("POS="+pos[n]+"_"+label,1.0,dataAlphabet);
	    fv = fv.add("POS="+pos[n]+"_WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
	    if(n > 0) {
		fv = fv.add("POS-1="+pos[n-1]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n] + "_POS-1="+pos[n-1]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS-1="+pos[n-1]+"_WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n] + "_POS-1="+pos[n-1]+"_WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
	    }
	    if(n > 1) {
		fv = fv.add("POS-2="+pos[n-2]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n] + "_POS-2="+pos[n-2]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS-2="+pos[n-2]+"_WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n] + "_POS-2="+pos[n-2]+"_WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n] + "_POS-1="+pos[n-1] + "_POS-2="+pos[n-2]+"_"+label,1.0,dataAlphabet);
	    }
	    if(n < toks.length-1) {
		fv = fv.add("POS+1="+pos[n+1]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n]+"_POS+1="+pos[n+1]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS+1="+pos[n+1]+"_WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n]+"_POS+1="+pos[n+1]+"_WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
	    }
	    if(n < toks.length-2) {
		fv = fv.add("POS+2="+pos[n+2]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n]+"_POS+2="+pos[n+2]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS+2="+pos[n+2]+"_WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n]+"_POS+2="+pos[n+2]+"_WORD="+toksL[n]+"_"+label,1.0,dataAlphabet);
		fv = fv.add("POS="+pos[n] + "_POS+1="+pos[n+1] + "_POS+2="+pos[n+2]+"_"+label,1.0,dataAlphabet);
	    }
	    if(n > 0 && n < toks.length-1)
		fv = fv.add("POS="+pos[n] + "_POS+1="+pos[n+1] + "_POS-1="+pos[n-1]+"_"+label,1.0,dataAlphabet);
	}

	return fv;
    }
	
    private FeatureVector createFeatureVectorA(String[] toks, String[] pos,
					       int n, String label,
					       FeatureVector fv) {

	// tag triples
	String tm2 = n >= 2 && pos != null ? pos[n-2] : "<SP>";
	String tm1 = n >= 1 && pos != null ? pos[n-1] : "<SP>";
	String tm0 = pos != null ? pos[n] : "<TP>";
	String tp2 = n < pos.length-2 && pos != null ? pos[n+2] : "<EP>";
	String tp1 = n < pos.length-1 && pos != null ? pos[n+1] : "<EP>";
	
	String wm2 = n >= 2 ? toks[n-2] : "<SW>";
	String wm1 = n >= 1 ? toks[n-1] : "<SW>";
	String wm0 = toks[n];
	String wp2 = n < toks.length-2 ? toks[n+2] : "<EW>";
	String wp1 = n < toks.length-1 ? toks[n+1] : "<EW>";
	
	String wsin0 = "W0="+wm2;
	String wsin1 = "W1="+wm1;
	String wsin2 = "W2="+wm0;
	String wsin3 = "W3="+wp1;
	String wsin4 = "W4="+wp2;
	
	String tsin0 = "T0="+tm2;
	String tsin1 = "T1="+tm1;
	String tsin2 = "T2="+tm0;
	String tsin3 = "T3="+tp1;
	String tsin4 = "T4="+tp2;
	
	String wbin0 = "WW0="+wm2+"_"+wm1;
	String wbin1 = "WW1="+wm1+"_"+wm0;
	String wbin2 = "WW2="+wm0+"_"+wp1;
	String wbin3 = "WW3="+wp1+"_"+wp2;
	
	String tbin0 = "TT0="+tm2+"_"+tm1;
	String tbin1 = "TT1="+tm1+"_"+tm0;
	String tbin2 = "TT2="+tm0+"_"+tp1;
	String tbin3 = "TT3="+tp1+"_"+tp2;
	
	String ttrip0 = "TTT0="+tm2+"_"+tm1+"_"+tm0;
	String ttrip1 = "TTT1="+tm1+"_"+tm0+"_"+tp1;
	String ttrip2 = "TTT2="+tm0+"_"+tp1+"_"+tp2;
	
	fv = fv.add(wsin0+"_"+label,1.0,dataAlphabet);
	fv = fv.add(wsin1+"_"+label,1.0,dataAlphabet);
	fv = fv.add(wsin2+"_"+label,1.0,dataAlphabet);
	fv = fv.add(wsin3+"_"+label,1.0,dataAlphabet);
	fv = fv.add(wsin4+"_"+label,1.0,dataAlphabet);
	
	fv = fv.add(tsin0+"_"+label,1.0,dataAlphabet);
	fv = fv.add(tsin1+"_"+label,1.0,dataAlphabet);
	fv = fv.add(tsin2+"_"+label,1.0,dataAlphabet);
	fv = fv.add(tsin3+"_"+label,1.0,dataAlphabet);
	fv = fv.add(tsin4+"_"+label,1.0,dataAlphabet);
	
	fv = fv.add(wbin0+"_"+label,1.0,dataAlphabet);
	fv = fv.add(wbin1+"_"+label,1.0,dataAlphabet);
	fv = fv.add(wbin2+"_"+label,1.0,dataAlphabet);
	fv = fv.add(wbin3+"_"+label,1.0,dataAlphabet);
	
	fv = fv.add(tbin0+"_"+label,1.0,dataAlphabet);
	fv = fv.add(tbin1+"_"+label,1.0,dataAlphabet);
	fv = fv.add(tbin2+"_"+label,1.0,dataAlphabet);
	fv = fv.add(tbin3+"_"+label,1.0,dataAlphabet);
	
	fv = fv.add(ttrip0+"_"+label,1.0,dataAlphabet);
	fv = fv.add(ttrip1+"_"+label,1.0,dataAlphabet);
	fv = fv.add(ttrip2+"_"+label,1.0,dataAlphabet);
	
	return fv;
    }
    private FeatureVector orthoFeatures(String word, String label, FeatureVector fv) {
	String CAPS = "[A-Z]";
	String LOW = "[a-z]";
	String CAPSNUM = "[A-Z0-9]";
	String ALPHA = "[A-Za-z]";
	String ALPHANUM = "[A-Za-z0-9]";
	String PUNT = "[,\\.;:?!]";

	if(word.matches(CAPS+".*"))
	    fv = fv.add("ORT=INITCAP_"+label,1.0,dataAlphabet);
	if(word.matches(CAPS+LOW+"*"))
	    fv = fv.add("ORT=CAPS_"+label,1.0,dataAlphabet);
	if(word.matches(CAPS+"+"))
	    fv = fv.add("ORT=ALLCAPS_"+label,1.0,dataAlphabet);
	if(word.matches("[ivxdlcm]+|[IVXDLCM]+"))
	    fv = fv.add("ORT=ROMAN_"+label,1.0,dataAlphabet);
	if(word.matches("\\.\\.+"))
	    fv = fv.add("ORT=MULTIDOT_"+label,1.0,dataAlphabet);
	if(word.matches("[^\\.]+.*\\."))
	    fv = fv.add("ORT=ENDDOT_"+label,1.0,dataAlphabet);
	if(word.matches(ALPHANUM+"+-"+ALPHANUM+"*"))
	    fv = fv.add("ORT=DASH_"+label,1.0,dataAlphabet);
	if(word.matches("[A-Z][A-Z\\.]*\\.[A-Z\\.]*"))
	    fv = fv.add("ORT=ACRO_"+label,1.0,dataAlphabet);
	if(word.matches(CAPS+"\\."))
	    fv = fv.add("ORT=INIT_"+label,1.0,dataAlphabet);
	if(word.matches(ALPHA))
	    fv = fv.add("ORT=SING_"+label,1.0,dataAlphabet);
	if(word.matches(CAPS))
	    fv = fv.add("ORT=CAP1_"+label,1.0,dataAlphabet);
	if(word.matches(PUNT))
	    fv = fv.add("ORT=PUNC_"+label,1.0,dataAlphabet);
	
	return fv;
    }
	
    private FeatureVector createFeatureVector(String[] toks, String[] pos, String[] labels,
					      FeatureVector fv) {

	int prev = -1;
	for(int i = 0; i < toks.length; i++) {
	    if(labels[i].equals("I")) {
		if(prev != -1)
		    fv = createFeatureVector(toks,pos,prev,i,fv);
		prev = i;
	    }
	}

	return fv;
    }
	
    public void closeAlphabets() {
	dataAlphabet.stopGrowth();
	DiscontinuousInstance.setDataAlphabet(dataAlphabet);
    }

    private void createForest(DiscontinuousInstance inst, ObjectOutputStream out) {
	String[] toks = inst.getInput().sentence;
	String[] pos = inst.getInput().pos;

	try {
				
	    for(int i = 0; i < toks.length; i++) {
		for(int j = i+1; j < toks.length; j++) {
		    FeatureVector nfv = 
			createFeatureVector(toks,pos,i,j,new FeatureVector(-1,-1.0,null));
		    nfv.sort();
		    
		    for(FeatureVector curr = nfv; curr.next!= null; curr = curr.next)
			if(curr.index >= 0)
			    out.writeInt(curr.index);
		    out.writeInt(-1);
		}
	    }
	    
	    out.writeInt(-2);
	    	    
	} catch (IOException e) {}
    }		
}
