package struct.discontinuous;

import struct.types.*;
import struct.alg.*;

import gnu.trove.*;

import java.io.*;
import java.text.DecimalFormat;
import java.util.HashMap;

public class DiscontinuousEvaluator implements Evaluator {


    public void evaluate(Instance[] data, Predictor pred) {
	System.out.println("Not implemented.");
    }

    public void evaluate(Instance[] data, Predictor predictor, String featureFile) throws IOException {
	
	int guessed = 0;
	int real = 0;
	int correct = 0;

	int g_dis = 0;
	int r_dis = 0;
	int c_dis = 0;

	
	System.out.println("Evaluating ... ");

	ObjectInputStream in = new ObjectInputStream(new FileInputStream(featureFile));

	for(int k = 0; k < data.length; k++) {

	    Instance inst = data[k];
	    Features feats = data[k].getFeatures(in);

	    DiscontinuousPrediction prediction = 
		((DiscontinuousPredictor)predictor).decode(inst,feats,true);

	    String[][] act_labels = ((DiscontinuousLabel)inst.getLabel()).labels;
	    String[][] pred_labels = ((DiscontinuousLabel)prediction.getBestLabel()).labels;

	    HashMap act = ((DiscontinuousLabel)inst.getLabel()).getLabelMap();
	    HashMap pred = ((DiscontinuousLabel)prediction.getBestLabel()).getLabelMap();//NoSubset();

	    guessed += pred.size();
	    real += act.size();

	    Object[] keys = pred.keySet().toArray();
	    for(int i = 0; i < keys.length; i++) {
		String p = (String)keys[i];
		boolean isDis = p.matches("[IO ]+I[ ][O ]+[ ]I[IO ]+");
		if(isDis) g_dis++;
		if(act.containsKey(keys[i])) {
		    correct++;
		    if(isDis)
			c_dis++;
		}
	    }
	    keys = act.keySet().toArray();
	    for(int i = 0; i < keys.length; i++) {
		String p = (String)keys[i];
		boolean isDis = p.matches("[IO ]+I[ ][O ]+[ ]I[IO ]+");
		if(isDis) r_dis++;
	    }

	    

	}

	in.close();
	DecimalFormat nf = new DecimalFormat();
	nf.setMaximumFractionDigits(5);
 
	System.out.println("Total:");
	System.out.println("\tGuessed: " + guessed + ", Actual: " 
			   + real + ", Correct: " + correct);
	double prec = (double)correct/guessed;
	double rec = (double)correct/real;
	double fmeas = (2*prec*rec)/(prec+rec);
	System.out.println("\tPrecision: " + nf.format(prec)
			   + ", Recall: " + nf.format(rec) 
			   + ", F-meas: " + nf.format(fmeas));

	System.out.println("Discontinuous:");
	System.out.println("\tGuessed: " + g_dis + ", Actual: " 
			   + r_dis + ", Correct: " + c_dis);
	prec = (double)c_dis/g_dis;
	rec = (double)c_dis/r_dis;
	fmeas = (2*prec*rec)/(prec+rec);
	System.out.println("\tPrecision: " + nf.format(prec)
			   + ", Recall: " + nf.format(rec) 
			   + ", F-meas: " + nf.format(fmeas));

	System.out.println("done.");
    }

}
