package struct.discontinuous;

import struct.types.*;

import java.io.*;

public class DiscontinuousInstance implements Instance {

    protected DiscontinuousInput input;
    protected DiscontinuousLabel label;

    protected static Alphabet dataAlphabet;

    public DiscontinuousInstance() {}
    
    public DiscontinuousInstance(String[] sentence, String[] pos,
				 String[][] labels, FeatureVector[] fv) {
	
	input = new DiscontinuousInput(sentence,pos);

	label = new DiscontinuousLabel(labels,fv);

    }
    
    public DiscontinuousInput getInput() { return input; }

    public DiscontinuousLabel getLabel() { return label; }

    public DiscontinuousFeatures getFeatures() { return null; }

    public DiscontinuousFeatures getFeatures(ObjectInputStream in) throws IOException {
	String[] toks = input.sentence;

	FeatureVector[][] fv_ij = new FeatureVector[toks.length][toks.length];
	FeatureVector[] fv_j = new FeatureVector[toks.length];

	for(int i = 0; i < toks.length; i++) {
	    for(int j = i+1; j < toks.length; j++) {
		fv_ij[i][j] = new FeatureVector(-1,-1.0,null);
		int n = in.readInt();
		while(n != -1) {
		    fv_ij[i][j] = new FeatureVector(n,1.0,fv_ij[i][j]);
		    n = in.readInt();
		}
	    }
	}
	
	if(in.readInt() != -2) { System.out.println("Error reading forest 1"); System.exit(0); }
	
	for(int j = 0; j < toks.length; j++)
	    fv_j[j] = new FeatureVector(-1,-1.0,null);	
	
	return new DiscontinuousFeatures(fv_ij,fv_j);
	    
    }

    public static void setDataAlphabet(Alphabet dataAlphabet1) {
	dataAlphabet = dataAlphabet1;
    }
    
    public static Alphabet getDataAlphabet() {
	return dataAlphabet;
    }
    
}
