package struct.discontinuous;

import struct.alg.*;
import struct.types.*;
import struct.solver.*;

import java.util.*;

public class DiscontinuousUpdator implements OnlineUpdator {

    public void update(Instance inst, Features feats, Predictor predictor, double avg_upd) {
	DiscontinuousLabel pred = ((DiscontinuousPrediction)predictor.decode(inst,feats)).getBestLabel();
	DiscontinuousLabel act = (DiscontinuousLabel)inst.getLabel();

	ArrayList corr = new ArrayList();
	for(int i = 0; i < act.fv.length; i++)
	    corr.add(act.fv[i]);

	HashMap actMap = act.getLabelMap();
	HashMap predMap = new HashMap();;

	ArrayList wrong = new ArrayList();
	for(int i = 0; i < pred.fv.length; i++) {
	    String[] p_labs = pred.labels[i];
	    String res = "";
	    for(int j = 0; j < p_labs.length; j++)
		res += p_labs[j] + " ";
	    res = res.trim();
	    if(!actMap.containsKey(res) && !predMap.containsKey(res)) {
		wrong.add(pred.fv[i]);
		predMap.put(res,"");
	    }
	}

	FeatureVector[] dist = new FeatureVector[corr.size() + wrong.size()];
	double[] b = new double[corr.size() + wrong.size()];
	int cnt = 0;
	for(int i = 0; i < corr.size(); i++) {
		FeatureVector corrFV = (FeatureVector)corr.get(i);
		FeatureVector zero = new FeatureVector(-1,-1.0,null);
		double lam_dist = predictor.score(corrFV)-predictor.score(zero);
		b[cnt] = 1.0 - lam_dist;
		dist[cnt] = FeatureVector.getDistVector(corrFV,zero);
		cnt++;
	}
        for(int i = 0; i < wrong.size(); i++) {
                FeatureVector wrongFV = (FeatureVector)wrong.get(i);
                FeatureVector zero = new FeatureVector(-1,-1.0,null);
                double lam_dist = predictor.score(zero)-predictor.score(wrongFV);
                b[cnt] = 1.0 - lam_dist;
                dist[cnt] = FeatureVector.getDistVector(zero,wrongFV);
                cnt++;
        }


	// update the weights
	double[] alpha = QPSolver.hildreth(dist,b);

	double[] weights = predictor.weights;
	double[] avg_weights = predictor.avg_weights;

	FeatureVector fv  = null;
	for(int k = 0; k < dist.length; k++) {
	    fv = dist[k];				
	    for(FeatureVector curr = fv; curr != null; curr = curr.next) {
		if(curr.index < 0)
		    continue;
		weights[curr.index] += alpha[k]*curr.value;
		avg_weights[curr.index] += avg_upd*alpha[k]*curr.value;
	    }
	}

    }


}
