package struct.relation;

import java.io.*;
import java.util.*;

public class RelationExtractorComplexCreator {
    public static String[] createGroups(String[] bin_predictions) {
	ArrayList groups = new ArrayList();
	for(int i = 0; i < bin_predictions.length; i++) {
	    String line = bin_predictions[i];
	    String toks[] = line.split("\t");
	    String v1 = toks[0] +" " +toks[2] + " " + toks[3];
	    String v2 = toks[1] +" " +toks[2] + " " + toks[4];
	    double w = Double.parseDouble(toks[5]);
	    double nw = Double.parseDouble(toks[6]);
	    boolean inc = toks[7].equals("true") ? true : false;
	    if(inc) {
		ArrayList edges = new ArrayList();
		edges.add(new Edge(v1,v2,toks[2],w,nw,inc));
		Group g = new Group(edges);
		if(g.canBeExtended())
		    groups.add(g);
	    }
	}

	groups = sort(groups);

	long start = System.currentTimeMillis();

	//System.out.println("merging tri groups ...");
	ArrayList tri_groups = new ArrayList();
	for(int i = 0; i < groups.size(); i++) {
	    Group g1 = (Group)groups.get(i);
	    for(int j = i+1; j < groups.size(); j++) {
		Group g2 = (Group)groups.get(j);
                if(!g1.sameGroup(g2))
                    break;
 		Group gn = g1.merge(g2);
		if(!gn.canBeExtended() || !g1.shareVert(g2))
		    continue;
		for(int k = j+1; k < groups.size(); k++) {
		    Group g3 = (Group)groups.get(k);
                    if(!g2.sameGroup(g3))
                        break;
		    if(!gn.shareVert(g2))
			continue;
 		    Group g = g3.merge(gn);
		    if(g.numVerts() == 3 && g.connected() && g.canBeExtended())
			tri_groups.add(g);
		}
	    }
	}
	//System.out.println(" done");
	tri_groups = sort(tri_groups);
	
	//System.out.println("merging six groups ...");
	ArrayList six_groups = new ArrayList();
	for(int k = 0; k < groups.size(); k++) {
	    Group g3 = (Group)groups.get(k);
	    boolean found = false;
	    for(int i = 0; i < tri_groups.size(); i++) {
		Group g1 = (Group)tri_groups.get(i);
		boolean g13 = g1.sameGroup(g3);
                if(!g13 && found)
                    break;
                 if(!g13 || g3.isSubset(g1) || !g1.shareVert(g3))
                    continue;
                found = true;
  		Group gn = g1.merge(g3);
		if(!gn.canBeExtended())
		    continue;
		for(int j = i+1; j < tri_groups.size(); j++) {
		    Group g2 = (Group)tri_groups.get(j);
                    if(!g1.sameGroup(g2))
                        break;
		    if(g2.isSubset(g3) || !gn.shareVert(g2))
			continue;
 		    Group g = g2.merge(gn);
		    if(g.numVerts() == 4 && g.connected() && g.canBeExtended())
			six_groups.add(g);
		}
	    }
	}
	//System.out.println(" done");
	six_groups = sort(six_groups);
	
	//System.out.println("Removing low prob groups");
	ArrayList new_groups = new ArrayList();
	for(int i = 0; i < groups.size(); i++) {
	    Group g1 = (Group)groups.get(i);
	    if(g1.weight >= 0.5)
		new_groups.add(g1);
	}
	groups = new_groups;

	new_groups = new ArrayList();
	for(int i = 0; i < tri_groups.size(); i++) {
	    Group g1 = (Group)tri_groups.get(i);
	    if(g1.weight >= 0.5)
		new_groups.add(g1);
	}
	tri_groups = new_groups;

	new_groups = new ArrayList();
	for(int i = 0; i < six_groups.size(); i++) {
	    Group g1 = (Group)six_groups.get(i);
	    if(g1.weight >= 0.5)
		new_groups.add(g1);
	}
	six_groups = new_groups;

	//System.out.println("Removing subsets");
	ArrayList good = new ArrayList();
	for(int i = 0; i < groups.size(); i++) {
	    Group g1 = (Group)groups.get(i);
	    boolean isSub = false;
	    for(int j = i+1; j < groups.size() && !isSub; j++) {
		Group g2 = (Group)groups.get(j);
		if(g2.isSubset(g1))
		    isSub = true;
	    }
	    for(int j = 0; j < tri_groups.size() && !isSub; j++) {
		Group g2 = (Group)tri_groups.get(j);
		if(g2.isSubset(g1))
		    isSub = true;
	    }
	    for(int j = 0; j < six_groups.size() && !isSub; j++) {
		Group g2 = (Group)six_groups.get(j);
		if(g2.isSubset(g1))
		    isSub = true;
	    }
		    
	    if(!isSub)
		good.add(g1);

	}

	for(int i = 0; i < tri_groups.size(); i++) {
	    Group g1 = (Group)tri_groups.get(i);
	    boolean isSub = false;
	    for(int j = i+1; j < tri_groups.size(); j++) {
		Group g2 = (Group)tri_groups.get(j);
		if(g2.isSubset(g1))
		    isSub = true;
	    }
	    for(int j = 0; j < six_groups.size(); j++) {
		Group g2 = (Group)six_groups.get(j);
		if(g2.isSubset(g1))
		    isSub = true;
	    }
		    
	    if(!isSub)
		good.add(g1);

	}
	
	for(int i = 0; i < six_groups.size(); i++) {
	    Group g1 = (Group)six_groups.get(i);
	    boolean isSub = false;
	    for(int j = i+1; j < six_groups.size(); j++) {
		Group g2 = (Group)six_groups.get(j);
		if(g2.isSubset(g1))
		    isSub = true;
	    }		    
	    if(!isSub)
		good.add(g1);

	}

	long end = System.currentTimeMillis();

	String[] result = new String[good.size()];
	for(int i = 0; i < good.size(); i++) {
	    Group g = (Group)good.get(i);
	    result[i] = g.toString().trim();
	}
	return result;
    }

    public static ArrayList sort(ArrayList list) {
	Object[] tmp = list.toArray();
	Arrays.sort(tmp);
	ArrayList result = new ArrayList();
	for(int i = 0; i < tmp.length; i++)
	    result.add(tmp[i]);
	return result;
    }

    public static class Group implements Comparable {
	public ArrayList edges;
	public double weight;
	public double neg_weight;
	public String file_id;
	public boolean extended;
	public int numV;
	public HashMap vertMap;

	public Group(ArrayList edges) {
	    this.edges = edges;

	    double total = 0.0;
	    double neg_total = 0.0;
	    
	    for(int i = 0; i < edges.size(); i++) {
		Edge e = ((Edge)edges.get(i));
		total += 1.0/e.weight;
		neg_total += 1.0/e.neg_weight;
	    }
	    this.weight = edges.size()/total;
	    this.neg_weight = edges.size()/neg_total;

	    for(int i = 0; i < 6-edges.size(); i++)
		total += 1.0;
	    extended = 6/total >= 0.5;

	    vertMap = new HashMap();
	    for(int i = 0; i < edges.size(); i++) {
		Edge e = (Edge)edges.get(i);
		vertMap.put(e.v1,"");
		vertMap.put(e.v2,"");
	    }

	    numV = vertMap.size();

	    file_id = ((Edge)edges.get(0)).file_id;

	}

	public boolean sameGroup(Group g) {
	    return file_id.equals(g.file_id);
	}

	// return true if a group can be extended to make a valid clique
	public boolean canBeExtended() {
	    return extended;
	}

	// return true if g is a subset of this group
	public boolean isSubset(Group g) {
	    for(int i = 0; i < g.edges.size(); i++) {
		Edge e1 = (Edge)g.edges.get(i);
		boolean found = false;

		for(int j = 0; !found && j < edges.size(); j++) {
		    Edge e2 = (Edge)edges.get(j);
		    if(e1.equals(e2))
			found = true;
		}
		if(!found)
		    return false;
	    }
	    return true;
	}

	// return true if g is a subset of this group
	public boolean shareVert(Group g) {
	    for(int i = 0; i < g.edges.size(); i++) {
		Edge e = (Edge)g.edges.get(i);
		if(vertMap.containsKey(e.v1) || vertMap.containsKey(e.v2))
		    return true;
	    }
	    return false;
	}

	public Group merge(Group g) {
	    ArrayList edges_new = new ArrayList();
	    HashMap already = new HashMap();
	    for(int i = 0; i < edges.size(); i++) {
		Edge e = (Edge)edges.get(i);
		edges_new.add(e);
		already.put(e.v1+" "+e.v2,"");
		already.put(e.v2+" "+e.v1,"");
	    }
	    for(int i = 0; i < g.edges.size(); i++) {
		Edge e = (Edge)g.edges.get(i);
		if(!already.containsKey(e.v1+" "+e.v2))
		    edges_new.add(e);
	    }
	    return new Group(edges_new);
	}

	public boolean connected() {
	    HashMap edgeMap = new HashMap();
	    for(int i = 0; i < edges.size(); i++) {
		Edge e = (Edge)edges.get(i);
		edgeMap.put(e.v1 + "\t" + e.v2,"");
		edgeMap.put(e.v2 + "\t" + e.v1,"");
	    }

	    if(vertMap.size() > 4)
		return false;

	    // ensure all edges present
	    Object[] v1s1 = vertMap.keySet().toArray();
	    String[] v1s = new String[v1s1.length];
	    for(int i = 0; i < v1s.length; i++)
		v1s[i] = (String)v1s1[i];

	    for(int i = 0; i < v1s.length; i++) {
		String vrt1 = v1s[i];
		for(int j = i+1; j < v1s.length; j++) {
		    if(j == i) continue;
		    String vrt2 = v1s[j];
		
		    if(!edgeMap.containsKey(vrt1+"\t"+vrt2)) {
			return false;
		    }
		}
	    }
	    return true;
	}

	public int numVerts() {
	    return numV;
	}

	public String toString() {
	    Object[] keys = vertMap.keySet().toArray();
	    String file = ((String)keys[0]).split(" ")[1];
	    String[] ids = new String[keys.length];
	    String[] types = new String[keys.length];
	    for(int i = 0; i < keys.length; i++) {
		ids[i] = ((String)keys[i]).split(" ")[0];
		types[i] = ((String)keys[i]).split(" ")[2];
	    }
	    //Arrays.sort(ids);
	    String result = "";
	    for(int i = 0; i < ids.length; i++)
		result += ids[i] + "\t";
	    result += file + "\t";
	    for(int i = 0; i < types.length; i++)
		result += types[i] + "\t";
	    result += weight;
	    return result.trim();
	}

	public int compareTo(Object o) {
	    Group g = (Group)o;
	    return file_id.compareTo(g.file_id);
	}

    }


    public static class Edge {

	public String v1, v2;
	public double weight, neg_weight;
	public String file_id;
	public boolean include;
	public Edge(String v1, String v2, String file_id,
		    double weight, double neg_weight, boolean include) {
	    this.v1 = v1;
	    this.v2 = v2;
	    this.weight = weight;
	    this.neg_weight = neg_weight;
	    this.file_id = file_id;
	    this.include = include;
	}

	public boolean equals(Edge e) {
	    return (v1.equals(e.v1) && v2.equals(e.v2)) 
		|| (v1.equals(e.v2) && v2.equals(e.v1));
	}

	public String toString() {
	    return v1 + "\t" + v2;
	}

    }

}
