package struct.relation;

import struct.types.*;

import java.io.*;
import java.util.*;

public class RelationExtractorInstance implements Instance {

    public RelationExtractorInput input;
    public RelationExtractorLabel label;
    public String[] lines;

    public RelationExtractorInstance(RelationExtractorInput input, RelationExtractorLabel label,
				     String[] lines) {
	this.input = input;
	this.label = label;
	this.lines = lines;
    }

    public RelationExtractorInput getInput() { return input; }
    
    public RelationExtractorLabel getLabel() { return label; }

    public RelationExtractorFeatures getFeatures() { 
	return new RelationExtractorFeatures(input.n_fv,input.y_fv);
    }

    public RelationExtractorFeatures getFeatures(ObjectInputStream in) throws IOException {
	return getFeatures();
    }

    public static RelationExtractorInstance[] split10Train(RelationExtractorInstance[] instances, int n) {
	ArrayList lst = new ArrayList();
	for(int i = 0; i < instances.length; i++) {
	    if(i % 10 != n)
		lst.add(instances[i]);
	}
	RelationExtractorInstance[] result = new RelationExtractorInstance[lst.size()];
	for(int i = 0; i < result.length; i++)
	    result[i] = (RelationExtractorInstance)lst.get(i);
	return result;
    }
    public static RelationExtractorInstance[] split10Test(RelationExtractorInstance[] instances, int n) {
	ArrayList lst = new ArrayList();
	for(int i = 0; i < instances.length; i++) {
	    if(i % 10 == n)
		lst.add(instances[i]);
	}
	RelationExtractorInstance[] result = new RelationExtractorInstance[lst.size()];
	for(int i = 0; i < result.length; i++)
	    result[i] = (RelationExtractorInstance)lst.get(i);
	return result;
    }

    public String toString() {
	String result = "";
	for(int i = 0; i < lines.length; i++) {
	    result += lines[i] + "\n\n";
	}
	return result;
    }

    public static RelationExtractorInstance[] generateBinary(RelationExtractorInstance[] data) {
	ArrayList result = new ArrayList();
	for(int k = 0; k < data.length; k++) {
	    RelationExtractorInstance inst = data[k];
	    
	    RelationExtractorInput inpt = inst.getInput();
	    RelationExtractorLabel label = inst.getLabel();
	    for(int i = 0; i < inst.lines.length; i++) {
		String[] e1 = new String[]{inpt.e1[i]};
		String[] e2 = new String[]{inpt.e2[i]};
		String[] t1 = new String[]{inpt.t1[i]};
		String[] t2 = new String[]{inpt.t2[i]};
		String[] file = new String[]{inpt.file[i]};
		FeatureVector[] n_fv = new FeatureVector[]{inpt.n_fv[i]};
		FeatureVector[] y_fv = new FeatureVector[]{inpt.y_fv[i]};
		String[] bin_labels = new String[]{label.bin_labels[i]};
		FeatureVector[] all_fv = new FeatureVector[]{label.all_fv[i]};
		String[] labels = new String[0];
		String[] n_lines = new String[]{inst.lines[i]};
		RelationExtractorInput n_inpt = new RelationExtractorInput(e1,e2,t1,t2,file,n_fv,y_fv); 
		RelationExtractorLabel n_label = new RelationExtractorLabel(labels,bin_labels,all_fv); 
		result.add(new RelationExtractorInstance(n_inpt,n_label,n_lines));
	    }
	}
	RelationExtractorInstance[] res = new RelationExtractorInstance[result.size()];
	for(int i = 0; i < res.length; i++)
	    res[i] = (RelationExtractorInstance)result.get(i);
	return res;
    }

}
