package struct.relation;

import java.io.*;
import java.util.*;

public class RelationExtractorInstanceGenerator {

    public static String filename = "";

    public static BufferedWriter outp;

    public static String[][] generate(BufferedReader in) throws IOException {

	Sentence[] sentences = new Sentence[0];
	Entity[] entities = new Entity[0];
	Relation[] relations = new Relation[0];
	HashMap tokens = new HashMap();
	HashMap ents = new HashMap();

	String line = in.readLine();
	while(line != null) {
	    if(line.startsWith("filename")) {
		String[][] r = null;
		if(sentences != null && sentences.length > 0)
		    r = createInstance(sentences,entities,relations);
		filename = line;
		return r;
	    }
	    else if(line.startsWith("<sentences>")) {
		sentences = new Sentence[Integer.parseInt(line.split(" ")[1])];
		processSentences(in,sentences,tokens);
	    }
	    else if(line.startsWith("<entities>")) {
		entities = new Entity[Integer.parseInt(line.split(" ")[1])];
		processEntities(in,entities,tokens,ents);
	    }
	    else if(line.startsWith("<relations>")) {
		relations = new Relation[Integer.parseInt(line.split(" ")[1])];
		processRelations(in,relations,ents);
	    }
		
	    line = in.readLine();
	}
	if(sentences != null && sentences.length > 0)
	    return createInstance(sentences,entities,relations);
	
	return null;
    }

    public static String[][] createInstance(Sentence[] sentences, Entity[] entities,
					    Relation[] relations) throws IOException {
	String[][] result = new String[2][];
	result[0] = outputGoldFull(relations);
	result[1] = outputRelationsBinary(sentences,entities,relations);
	return result;
    }

    public static String[] outputGoldFull(Relation[] relations) {
	
	LinkedList lst = new LinkedList();

	for(int i = 0; i < relations.length; i++) {
	    Relation r = relations[i];
	    Entity[] entities = r.entities;
	    String result = "";
	    if(entities.length < 2)
		continue;
	    for(int j = 0; j < entities.length; j++) {
		result += entities[j].id + "\t";
	    }
	    result += filename;//+entities[0].sentNum;
	    lst.add(result);
	}


	String[] res = new String[lst.size()];
	for(int i = 0; i < lst.size(); i++)
	    res[i] = (String)lst.get(i);
	return res;

    }

    public static String[] outputRelationsBinary(Sentence[] sentences,
						 Entity[] entities,
						 Relation[] relations) throws IOException {

	LinkedList res = new LinkedList();
	HashMap already = new HashMap();

	ArrayList[] ents_by_sent = new ArrayList[sentences.length];
	for(int i = 0; i < ents_by_sent.length; i++)
	    ents_by_sent[i] = new ArrayList();

	for(int i = 0; i < entities.length; i++)
	    ents_by_sent[entities[i].sentNum].add(entities[i]);

	for(int s = 0; s < ents_by_sent.length; s++) {
	    ArrayList ents = ents_by_sent[s];

	    for(int i = 0; i < ents.size(); i++) {
		Entity e1 = (Entity)ents.get(i);
		for(int j = i+1; j < ents.size(); j++) {
		    Entity e2 = (Entity)ents.get(j);

		    if(!e1.isSmaller(e2)) {
			Entity tmp = e2;
			e2 = e1;
			e1 = tmp;
		    }
			
		    if(e1.type.equals(e2.type))
			continue;

		    String result = (areRelated(e1,e2,relations) ? "Y" : "N") + "\t";
		    result += e1.text + "\t" + e2.text + "\t";
		    result += reduceType(e1.type) + "\t" + reduceType(e2.type) + "\t";

		    Token t1 = e1.getMax();
		    Token t2 = e2.getMin();
		    Token[] sent_toks = sentences[s].tokens;
		    ArrayList between = new ArrayList();
		    boolean inB = false;
		    boolean mT = false;
		    boolean mL = false;
		    boolean mSO = false;
		    boolean mSA = false;
		    for(int t = 0; t < sent_toks.length; t++) {
			if(sent_toks[t] == t2)
			    break;
			else if(sent_toks[t] == t1)
			    inB = true;
			else if(inB) {
			    between.add(sent_toks[t]);
			    if(sent_toks[t].isType)
				mT = true;
			    if(sent_toks[t].isLoc)
				mL = true;
			    if(sent_toks[t].isSO)
				mSO = true;
			    if(sent_toks[t].isSA)
				mSA = true;			    
			}
		    }

		    result += between.size() + "\t";
		    for(int k = 0; k < between.size(); k++) {
			Token t = (Token)between.get(k);
			result += t.text + "\t" + t.pos + "\t";
		    }

		    if(mT || mL || mSO || mSA)
			result += "HAS_ENT\t";
		    if(mT)
			result+= "HAS_TYPE\t";
		    if(mL)
			result+= "HAS_LOC\t";
		    if(mSO)
			result+= "HAS_SO\t";
		    if(mSA)
			result+= "HAS_SA\t";

		    result += e1.id + "\t" + e2.id + "\t" + filename;

		    if(!already.containsKey(result)) {
			res.add(result);
			already.put(result,"");
		    }
		    
		}
	    }
	}
	String[] fin = new String[res.size()];
	for(int i = 0; i < fin.length; i++) {
	    fin[i] = (String)res.get(i);
	}
	return fin;
    }

    public static boolean areRelated(Entity e1, Entity e2, Relation[] relations) {
	if(relations == null)
	    return false;
	for(int i = 0; i < relations.length; i++) {
	    Relation r = relations[i];
	    if(r.containsEnt(e1) && r.containsEnt(e2))
		return true;
	}
	return false;
    }

    public static String reduceType(String s) {
	if(s.indexOf("type") > 0)
	    return "t";
	else if(s.indexOf("location") > 0)
	    return "l";
	else if(s.indexOf("original") > 0)
	    return "o";
	else if(s.indexOf("altered") > 0)
	    return "a";
	else {
	    System.out.println("Bad type: " + s);
	    System.exit(0);
	}
	return "";
    }

    public static void processSentences(BufferedReader in, Sentence[] sentences,
					HashMap tokenMap) throws IOException {
	in.readLine();
	int num = 0;
	while(num < sentences.length) {
	    String line = in.readLine();
	    ArrayList tokens = new ArrayList();
	    while(line.trim().length() > 0) {
		String[] toks = line.split("\t");
		Token t = new Token(toks[0],toks[1],toks[2],num);
		tokenMap.put(toks[0],t);
		tokens.add(t);
		line = in.readLine();
	    }
	    Token[] ts = new Token[tokens.size()];
	    for(int i = 0; i < ts.length; i++)
		ts[i] = ((Token)tokens.get(i));
	    sentences[num] = new Sentence(ts);
	    num++;
	}
    }

    public static void processEntities(BufferedReader in, Entity[] entities,
				       HashMap tokMap, HashMap ents) throws IOException {
	in.readLine();
	int num = 0;
	String line = in.readLine();
	while(num < entities.length) {
	    String[] toks = line.split("\t");
	    String[] ts = toks[3].split(",");
	    Arrays.sort(ts); // put in right order
	    Token[] tks = new Token[ts.length];
	    for(int i = 0; i < tks.length; i++) {
		tks[i] = (Token)tokMap.get(ts[i]);
		if(tks[i] == null) {
		    System.out.println(filename +" " + ts[i]);
		    System.exit(0);
		}
	    }
	    Entity e = new Entity(toks[0],toks[1],toks[2],tks);
	    ents.put(toks[0],e);
	    entities[num] = e;
	    line = in.readLine();
	    num++;
	}
	if(line.trim().length() > 0) { System.out.println("Bad Entities."); System.exit(0); }
    }

    public static void processRelations(BufferedReader in, Relation[] relations,
					HashMap entMap) throws IOException {
	in.readLine();
	int num = 0;
	String line = in.readLine();
	while(num < relations.length) {
	    String[] toks = line.split("\t");
	    String[] ents = toks[1].split(",");
	    Entity[] entities = new Entity[ents.length];
	    boolean validRelation = true;
	    for(int i = 0; i < entities.length; i++) {
		entities[i] = (Entity)entMap.get(ents[i]);
		if(entities[i] == null) {
		    validRelation = false;
		    break;
		}
	    }
	    if(validRelation)
		relations[num] = new Relation(toks[0],entities);
	    else {
		System.out.println("ERROR - bad entity."); System.exit(0);
		relations[num] = null;
	    }
	    line = in.readLine();
	    num++;
	}
	if(line.trim().length() > 0) { System.out.println("Bad Relations."); System.exit(0); }
    }

    public static class Token {
	public String id;
	public String text;
	public String pos;
	public int sentNum;
	public boolean isType, isLoc, isSO, isSA;
	public String entID;
	public Token(String id, String pos, String text, int sentNum) {
	    this.id = id;
	    this.pos = pos;
	    this.text = text;
	    this.sentNum = sentNum;
	    entID = "";
	}
    }

    public static class Sentence {
	public Token[] tokens;
	public Sentence(Token[] tokens) {
	    this.tokens = tokens;
	}
    }

    public static class Entity {
	public String id;
	public String text;
	public String type;
	public Token[] tokens;
	public int sentNum;
	public Entity(String id, String type, String text, Token[] tokens) {
	    this.id = id;
	    this.text = text;
	    this.type = type;
	    this.tokens = tokens;
	    for(int i = 0; i < tokens.length; i++) {
		if(type.equals("[variation-type]"))
		    this.tokens[i].isType = true;
		else if(type.equals("[variation-location]"))
		    this.tokens[i].isLoc = true;
		else if(type.equals("[variation-state-original]"))
		    this.tokens[i].isSO = true;
		else if(type.equals("[variation-state-altered]"))
		    this.tokens[i].isSA = true;
	    }
	    this.sentNum = tokens[0].sentNum;
	}
	public boolean isSmaller(Entity e2) {
	    int minE1 = Integer.parseInt(getMin().id);
	    int minE2 = Integer.parseInt(e2.getMin().id);
	    return minE1 < minE2;
	}
	public Token getMin() {
	    int minE1 = 999999999;
	    Token t = null;
	    for(int i = 0; i < tokens.length; i++)
		if(Integer.parseInt(tokens[i].id) < minE1) {
		    minE1 = Integer.parseInt(tokens[i].id);
		    t = tokens[i];
		}
	    return t;
	}
	public Token getMax() {
	    int maxE1 = -1;
	    Token t = null;
	    for(int i = 0; i < tokens.length; i++)
		if(Integer.parseInt(tokens[i].id) > maxE1) {
		    maxE1 = Integer.parseInt(tokens[i].id);
		    t = tokens[i];
		}
	    return t;
	}
    }

    public static class Relation {
	public String id;
	public Entity type, location, state_orig, state_alt;
	public Entity[] entities;
	public Relation(String id, Entity[] entities) {
	    this.id = id;
	    for(int i = 0; i < entities.length; i++) {
		if(entities[i] == null)
		    continue;
		if(entities[i].type.equals("[variation-type]"))
		    type = entities[i];
		else if(entities[i].type.equals("[variation-location]"))
		    location = entities[i];
		else if(entities[i].type.equals("[variation-state-original]"))
		    state_orig = entities[i];
		else if(entities[i].type.equals("[variation-state-altered]"))
		    state_alt = entities[i];
		else {
		    System.out.println("Bad entity type: " + entities[i].type);
		    System.exit(0);
		}
	    }
	    this.entities = entities;
	}

	public boolean containsEnt(Entity e1) {
	    if(type == e1 || location == e1 || state_orig == e1 || state_alt == e1)
		return true;
	    return false;
	}
	
    }
}
