package struct.relation;

import struct.types.*;

import java.util.*;

public class RelationExtractorLabel implements Label {
    private static boolean useRelationLoss = true;

    public String[] label;
    public String[] bin_labels;
    public FeatureVector[] all_fv;
    public FeatureVector fv;

    public RelationExtractorLabel(String[] label, String[] bin_labels,
				  FeatureVector[] all_fv) {

	this.label = label;
	this.bin_labels = bin_labels;
	this.all_fv = all_fv;
	fv = new FeatureVector(-1,-1.0,null);
	for(int i = 0; i < all_fv.length; i++)
	    fv = FeatureVector.cat(all_fv[i],fv);
    }


    public static void useRelationLoss(boolean b) { useRelationLoss = b; }

    // should all fv representations be collapsed?
    public FeatureVector getFeatureVectorRepresentation() { return fv; }

     // evaluates a prediction against this label
    public double loss(Label pred) {
	if(useRelationLoss)
	    return perRelLoss(pred);
	else
	    return perPairLoss(pred);
    }

    public double perPairLoss(Label pred) {
	String[] p_bin = ((RelationExtractorLabel)pred).bin_labels;

	int numErr = 0;
	for(int i = 0; i < p_bin.length; i++)
	    if(!p_bin[i].equals(bin_labels[i]))
		numErr++;
	
	return (double)numErr;
    }

   public double perRelLoss(Label pred) {
	double corr = 0; double guessed = 0; double total = 0;
	String[] pred_labels = ((RelationExtractorLabel)pred).label;
	String[] act_labels = label;
	
	HashMap corrMap = new HashMap();
	for(int j = 0; j < act_labels.length; j++) {
	    String line = act_labels[j];
	    //if(line == null) continue;
	    String[] toks = line.split("\t");
	    if(toks.length > 2) {
		Integer[] ids = new Integer[toks.length-1];
		for(int i = 0; i < ids.length; i++)
		    ids[i] = new Integer(Integer.parseInt(toks[i]));
		Arrays.sort(ids);
		line = "";
		for(int i = 0; i < ids.length; i++)
		    line += ids[i] + "\t";
		line += toks[toks.length-1];
		if(!corrMap.containsKey(line)) {
		    corrMap.put(line,"");
		    total++;
		}
	    }
	}
	
	
	for(int j = 0; j < pred_labels.length; j++) {
	    String line = pred_labels[j];
	    String[] toks = line.split("\t");
	    Integer[] ids = new Integer[((toks.length-2)/2)];
	    for(int i = 0; i < ids.length; i++)
		ids[i] = new Integer(Integer.parseInt(toks[i]));
	    Arrays.sort(ids);
	    line = "";
	    for(int i = 0; i < ids.length; i++)
		line += ids[i] + "\t";
	    line += toks[((toks.length-2)/2)];
	    guessed++;
	    if(corrMap.containsKey(line))
		corr++;
	}
	double prec = (double)corr/guessed;
	double rec = (double)corr/total;
	double fmeas = (2*prec*rec)/(prec+rec);
	return (double)(guessed-corr)+(total-corr);

    }

}

