package struct.relation;

import struct.alg.*;
import struct.types.*;

import java.io.*;
import gnu.trove.*;
import java.util.*;

public class RelationExtractorPredictor extends Predictor {

    public boolean useAvg;

    public RelationExtractorPredictor(int dimensions, boolean useAvg) {
	super(dimensions);
	this.useAvg = useAvg;
    }

    public void setAvg(boolean useAvg) {
	this.useAvg = useAvg;
    }
    
    public RelationExtractorPrediction decode(Instance inst, Features feats) {
	return decode(inst,feats,1);
    }

    public RelationExtractorPrediction decode(Instance inst, Features feats, int K) {
	RelationExtractorInstance instance = (RelationExtractorInstance)inst;
	RelationExtractorFeatures features = (RelationExtractorFeatures)feats;
	
	RelationExtractorInput input = instance.getInput();
	RelationExtractorLabel corrLabel = instance.getLabel();

	int num_bin = input.e1.length;
	TIntIntHashMap already = new TIntIntHashMap();
	int constraints = Math.min(K,num_bin+1);
	RelationExtractorLabel[] labs = new RelationExtractorLabel[constraints];
	double minErr = Double.POSITIVE_INFINITY;
	int whichOne = -1;
	int num = 0;
	for(int k = 0; k < constraints; k++) {
	    String[] bin_labels = new String[num_bin];
	    double[] bin_probs = new double[num_bin];
	    double[] neg_bin_probs = new double[num_bin];
	    FeatureVector[] bin_fv = new FeatureVector[num_bin];
	    String[] bin_lines = new String[num_bin];

	    double closest = Double.POSITIVE_INFINITY;
	    int which = -1;

	    for(int i = 0; i < num_bin && k != 0; i++) {
		FeatureVector y_fv = input.y_fv[i];
		FeatureVector n_fv = input.n_fv[i];
		if(score(y_fv) >= score(n_fv)) {
		    bin_labels[i] = "Y";
		    bin_fv[i] = y_fv;
		}
		else {
		    bin_labels[i] = "N";
		    bin_fv[i] = n_fv;
		}
		double s_y = Math.exp(score(y_fv));
		double s_n = Math.exp(score(n_fv));
		bin_probs[i] = s_y/(s_y+s_n);
		neg_bin_probs[i] = s_n/(s_y+s_n);

		double ratio = Math.max(bin_probs[i],neg_bin_probs[i]);
		if(ratio < closest && !already.contains(i)) { 
		    closest = ratio; 
		    which = i;
		}
	    }
	    already.put(which,0);	

	    if(which < 0 && k > 0)
		break;
	    num++;

	    for(int i = 0; i < num_bin; i++) {
		FeatureVector y_fv = input.y_fv[i];
		FeatureVector n_fv = input.n_fv[i];
		double s_y = Math.exp(score(y_fv));
		double s_n = Math.exp(score(n_fv));
		if(score(y_fv) >= score(n_fv) && i != which) {
		    bin_labels[i] = "Y";
		    bin_fv[i] = y_fv;
		}
		else if(score(y_fv) < score(n_fv) && i != which) {
		    bin_labels[i] = "N";
		    bin_fv[i] = n_fv;
		}
		else if(score(y_fv) < score(n_fv) && i == which) {
		    bin_labels[i] = "Y";
		    bin_fv[i] = y_fv;
		    s_y = 0.5; s_n = 0.5;
		}
		else if(score(y_fv) >= score(n_fv) && i == which) {
		    bin_labels[i] = "N";
		    bin_fv[i] = n_fv;
		    s_y = 0.49; s_n = 0.51;
		}
		bin_probs[i] = s_y/(s_y+s_n);
		neg_bin_probs[i] = s_n/(s_y+s_n);
		boolean include = useAvg ? true : bin_probs[i] >= neg_bin_probs[i];
		bin_lines[i] = input.e1[i]+"\t"+input.e2[i]+"\t"+
		    input.file[i]+"\t"+input.t1[i]+"\t"+input.t2[i]+"\t"+
		    bin_probs[i]+"\t"+neg_bin_probs[i] + "\t" + include;
	    }

	    // complex relation label
	    String[] label = RelationExtractorComplexCreator.createGroups(bin_lines);

	    //updateBins(label,bin_labels,bin_fv,input);
	    labs[k] = new RelationExtractorLabel(label,bin_labels,bin_fv);
	}

	if(num_bin == 0) {
	    labs = new RelationExtractorLabel[1];
	    labs[0] = new RelationExtractorLabel(new String[0],
						 new String[0],
						 new FeatureVector[]{new FeatureVector(-1,-1.0,null)});
	}
	return new RelationExtractorPrediction(labs);
    }

    private void updateBins(String[] label, String[] bin_labels, FeatureVector[] bin_fv, RelationExtractorInput inp) {
	for(int i = 0; i < label.length; i++) {
	    String[] toks = label[i].split("\t");
	    String[] ids = new String[((toks.length-2)/2)];
	    for(int j = 0; j < ids.length; j++)
		ids[j] = toks[j];
	    for(int j = 0; j < ids.length; j++) {
		String id1 = ids[j];
		for(int k = 0; k < ids.length; k++) {
		    String id2 = ids[k];
		    for(int l = 0; l < inp.e1.length; l++) {
			if((inp.e1[l].equals(id1) && inp.e2[l].equals(id2)) ||
			   (inp.e1[l].equals(id2) && inp.e2[l].equals(id1))) {
			    bin_labels[l] = "Y";
			    bin_fv[l] = inp.y_fv[l];
			    break;
			}
		    }

		}
	    }
		
	}
    }

}
