package struct.sequence;

import struct.types.*;

import java.io.*;

public class HandWritInstance extends SequenceInstance {

    public HandWritInstance(String[] sentence, String[] pos,
			    String[] tags, FeatureVector fv) {
	
	super(sentence,pos,tags,fv);
    }
    
    public HandWritInstance(String[] sentence,
			    String[] tags, FeatureVector fv) {
	super(sentence,tags,fv);
    }
    
    public SequenceFeatures getFeatures(ObjectInputStream in) throws IOException {

	String[] toks = input.sentence;
	
	FeatureVector fvs_ij[][][] = 
	    new FeatureVector[toks.length][tagAlphabet.size()][tagAlphabet.size()];
	FeatureVector fvs_j[][] = new FeatureVector[toks.length][tagAlphabet.size()];

	for(int k = 0; k < toks.length; k++) {
	    for(int i = 0; i < tagAlphabet.size(); i++) {
		for(int j = 0; j < tagAlphabet.size(); j++) {
		    FeatureVector prodFV = new FeatureVector(-1,-1.0,null);
		    
		    int indx = in.readInt();
		    while(indx != -1) {
			double val = in.readDouble();
			prodFV = new FeatureVector(indx,val,prodFV);
			indx = in.readInt();
		    }
		    
		    fvs_ij[k][i][j] = prodFV;
		    
		}
	    }
	}
	
	int i1 = in.readInt();
	if(i1 != -2) {
	    System.out.println("Feature Vector I/O problem 1:" + i1);
	    System.out.println(toks[1] + "\t" + tagAlphabet.size());
	    System.exit(0);
	}
	
	for(int k = 0; k < toks.length; k++) {
	    for(int i = 0; i < tagAlphabet.size(); i++) {
		FeatureVector prodFV = new FeatureVector(-1,-1.0,null);
		
		int indx = in.readInt();
		while(indx != -1) {
		    double val = in.readDouble();
		    prodFV = new FeatureVector(indx,val,prodFV);
		    indx = in.readInt();
		}
		
		fvs_j[k][i] = prodFV;
		
	    }
	}
	
	i1 = in.readInt();
	if(i1 != -3) {
	    System.out.println("Feature Vector I/O problem 2:" + i1);
	    System.out.println(toks[1] + "\t" + tagAlphabet.size() + "\t");
	    System.exit(0);
	}
	
	return new SequenceFeatures(fvs_ij,fvs_j);

    }

}
