package struct.sequence;

import struct.types.*;

import java.io.*;
import gnu.trove.*;
import java.util.*;

public abstract class SequenceDataManager implements DataManager {

    public Alphabet dataAlphabet;	
    public Alphabet tagAlphabet;	

    public SequenceDataManager()  {
	dataAlphabet = new Alphabet();
	tagAlphabet = new Alphabet();
    }

    public Alphabet getDataAlphabet() { return dataAlphabet; }
    public Alphabet getTagAlphabet() { return tagAlphabet; }

    public SequenceInstance[] readData(String file) throws IOException {
	return readData(file,true);
    }

    public SequenceInstance[] readData(String file, boolean createFeatureFile) throws IOException {
		
	BufferedReader in =
	    new BufferedReader(new InputStreamReader(new FileInputStream(file),"8859_2"));
	String line = in.readLine();
	String pos_line = in.readLine();
	String tags_line = in.readLine();
		
	LinkedList lt = new LinkedList();

	ObjectOutputStream out = 
	    createFeatureFile ? new ObjectOutputStream(new FileOutputStream(file+".feats")) : null;
		
	BufferedWriter crf = new BufferedWriter(new FileWriter(file+".crf"));

	int num1 = 0;
	System.out.println("Creating feature vectors and/or forests ...");
	while(line != null) {
	    if(num1 > 0 && num1 % 500 == 0)
		System.out.println("Creating Feature Vector Instance: " + num1
				   + ", Num Feats: " + dataAlphabet.size());

	    String[] toks = line.split("\t");
	    String[] pos = pos_line.split("\t");
	    String[] tags = tags_line.split("\t");
				
	    String[] toks_new = new String[toks.length+1];
	    String[] pos_new = new String[pos.length+1];
	    String[] tags_new = new String[tags.length+1];
	    toks_new[0] = "<init>";
	    pos_new[0] = "<init-POS>";
	    tags_new[0] = "O";
	    for(int i = 0; i < toks.length; i++) {
		toks_new[i+1] = normalize(toks[i]);
		pos_new[i+1] = pos[i];
		tags_new[i+1] = tags[i];
	    }
	    toks = toks_new;
	    pos = pos_new;
	    tags = tags_new;
			
	    LinkedList[] predicates = getPredicates(toks,pos);
	    FeatureVector fv = 
				createFeatureVector(predicates,tags,new FeatureVector(-1,-1.0,null));	
	    fv.sort();
	
	    SequenceInstance si = new SequenceInstance(toks,tags,fv);

	    if(createFeatureFile)
		createForest(si,predicates,out);
			
	    createCRF(predicates,tags,crf);

	    lt.add(si);
			
	    line = in.readLine();
	    pos_line = in.readLine();
	    tags_line = in.readLine();
	    num1++;
	}
		
	SequenceInstance[] si = new SequenceInstance[lt.size()];
	for(int i = 0; i < si.length; i++) {
	    si[i] = (SequenceInstance)lt.get(i);
	}

	if(createFeatureFile)
	    out.close();
		
	crf.close();
	return si;
		
    }

    public void createAlphabets(String file) throws IOException {
	createAlphabets(file,false);
    }

    public void createAlphabets(String file, boolean createUnsupported) throws IOException {
	System.out.print("Creating Entity Alphabet ... ");
	createTagAlphabet(file);
	System.out.println("done.");
	System.out.println("Num Labels: " + tagAlphabet.size());
	for(int i = 0; i < tagAlphabet.size(); i++)
	    System.out.println((String)tagAlphabet.lookupObject(i));

	System.out.print("Creating Data Alphabet ... ");

	BufferedReader in =
	    new BufferedReader(new InputStreamReader(new FileInputStream(file),"8859_2"));
	String line = in.readLine();
	String pos_line = in.readLine();
	String tags_line = in.readLine();
		
	int cnt = 0;
	while(line != null) {
			
	    String[] toks = line.split("\t");
	    String[] pos = pos_line.split("\t");
	    String[] tags = tags_line.split("\t");

	    String[] toks_new = new String[toks.length+1];
	    String[] pos_new = new String[pos.length+1];
	    String[] tags_new = new String[tags.length+1];
	    toks_new[0] = "<init>";
	    pos_new[0] = "<init-POS>";
	    tags_new[0] = "O";
	    for(int i = 0; i < toks.length; i++) {
		toks_new[i+1] = normalize(toks[i]);
		pos_new[i+1] = pos[i];
		tags_new[i+1] = tags[i];
	    }
	    toks = toks_new;
	    pos = pos_new;
	    tags = tags_new;
			
	    LinkedList[] predicates = getPredicates(toks,pos);
	    FeatureVector fv =
		createFeatureVector(predicates,tags,new FeatureVector(-1,-1.0,null));
	    
	    if(createUnsupported)
		createU(predicates);

	    line = in.readLine();
	    pos_line = in.readLine();
	    tags_line = in.readLine();
	    //System.out.println(cnt + ": " + dataAlphabet.size());
	    cnt++;
	}

	System.out.println("done.");
    }
	
    public void createU(LinkedList[] predicates) {
	for(int k = 0; k < predicates.length; k++) {
	    for(int i = 0; i < tagAlphabet.size(); i++) {
		for(int j = 0; j < tagAlphabet.size(); j++) {
		    FeatureVector nfv = 
			createFeatureVector(predicates[k],
					    (String)tagAlphabet.lookupObject(i),
					    (String)tagAlphabet.lookupObject(j),
					    new FeatureVector(-1,-1.0,null));			
		}
	    }
	}
	for(int k = 0; k < predicates.length; k++) {
	    for(int j = 0; j < tagAlphabet.size(); j++) {
		FeatureVector nfv = 
		    createFeatureVector(predicates[k],
					(String)tagAlphabet.lookupObject(j),
					new FeatureVector(-1,-1.0,null));
		
	    }
	}
    }
    
    public void createCRF(LinkedList[] predicates, String[] tags,
			  BufferedWriter out) throws IOException {

	for(int k = 1; k < predicates.length; k++) {
	    LinkedList preds = predicates[k];
	    String res = "";
	    for(int i = 0; i < preds.size(); i++)
		res += (String)preds.get(i) + " ";
	    res += tags[k];
	    out.write(res.trim() + "\n");
	}
	out.write("\n");
    }
    
    private void createTagAlphabet(String file) throws IOException {

	BufferedReader in = 
	    new BufferedReader(new InputStreamReader(new FileInputStream(file),"8859_2"));
	String line = in.readLine();
	String pos_line = in.readLine();
	String tags_line = in.readLine();
		
	while(line != null) {
			
	    String[] tags = tags_line.split("\t");

	    tagAlphabet.lookupIndex("O");
	    for(int i = 0; i < tags.length; i++) {
		tagAlphabet.lookupIndex(tags[i]);
	    }

	    line = in.readLine();
	    pos_line = in.readLine();
	    tags_line = in.readLine();
	}

    }


    public FeatureVector createFeatureVector(LinkedList predicates,
					    String prev, String next,
					    FeatureVector fv) {

	String s1 = prev+"_"+next;
	for(int j = 0; j < predicates.size(); j++) {
	    String pred = (String)predicates.get(j);
	    fv = fv.add(s1+"_"+pred,1.0,dataAlphabet);
	}
		
	return fv;
    }
	
    public FeatureVector createFeatureVector(LinkedList predicates,
					    String next,
					    FeatureVector fv) {

	String s2 = next;
	for(int j = 0; j < predicates.size(); j++) {
	    String pred = (String)predicates.get(j);
	    fv = fv.add(s2+"_"+pred,1.0,dataAlphabet);
	}
		
	return fv;
    }
	
    public FeatureVector createFeatureVector(LinkedList[] predicates,
					    String[] tags,
					    FeatureVector fv) {

	for(int i = 1; i < predicates.length; i++) {
	    fv = createFeatureVector(predicates[i],(i == 0 ? "O" : tags[i-1]),tags[i],fv);
	    fv = createFeatureVector(predicates[i],tags[i],fv);
	}

	return fv;
    }

    public abstract LinkedList[] getPredicates(String[] toks, String[] pos);
	
    public void closeAlphabets() {
	dataAlphabet.stopGrowth();
	tagAlphabet.stopGrowth();
	SequenceInstance.setTagAlphabet(tagAlphabet);
	SequenceInstance.setDataAlphabet(dataAlphabet);
    }

    public void createForest(SequenceInstance inst, LinkedList[] predicates, ObjectOutputStream out) {
	String[] toks = inst.getInput().sentence;

	try {
				
	    for(int k = 0; k < predicates.length; k++) {
		for(int i = 0; i < tagAlphabet.size(); i++) {
		    for(int j = 0; j < tagAlphabet.size(); j++) {
			FeatureVector nfv = 
			    createFeatureVector(predicates[k],
					       (String)tagAlphabet.lookupObject(i),
					       (String)tagAlphabet.lookupObject(j),
					       new FeatureVector(-1,-1.0,null));
			
			nfv.sort();

			for(FeatureVector curr = nfv; curr.next!= null; curr = curr.next)
			    if(curr.index >= 0)
				out.writeInt(curr.index);
			out.writeInt(-1);
		    }
		}
	    }
	    
	    out.writeInt(-2);
	    
	    for(int k = 0; k < predicates.length; k++) {
		for(int j = 0; j < tagAlphabet.size(); j++) {
		    FeatureVector nfv = 
			createFeatureVector(predicates[k],
					   (String)tagAlphabet.lookupObject(j),
					   new FeatureVector(-1,-1.0,null));
		    
		    nfv.sort();

		    for(FeatureVector curr = nfv; curr.next!= null; curr = curr.next)
			if(curr.index >= 0)
			    out.writeInt(curr.index);
		    out.writeInt(-1);
		}
	    }

	    out.writeInt(-3);
	    	    
	    
	} catch (IOException e) {}
    }
		
    public String normalize(String s) {
	if(s.matches("[0-9]+|[0-9]+\\.[0-9]+|[0-9]+[0-9,]+"))
	    return "<num>";
	
	return s;
    }	
}
