package struct.sequence;

import struct.types.*;
import struct.alg.*;

import gnu.trove.*;

import java.io.*;
import java.text.DecimalFormat;
 
public class SequenceEvaluator implements Evaluator {

    Alphabet tagAlphabet;

    public SequenceEvaluator(Alphabet tagAlphabet) {
	this.tagAlphabet = tagAlphabet;
    }

    public void evaluate(Instance[] data, Predictor pred) {
	System.out.println("Not implemented.");
    }

    public void evaluate(Instance[] data, Predictor predictor, String featureFile) throws IOException {
	TObjectIntHashMap sufMap = new TObjectIntHashMap();
	TIntObjectHashMap revSufMap = new TIntObjectHashMap();

	int nn = 0;
	for(int i = 0; i < tagAlphabet.size(); i++) {
	    String tag = (String)tagAlphabet.lookupObject(i);
	    String[] toks = tag.split("-");
	    if(toks.length > 1) {
		String suf = toks[1];
		if(!sufMap.contains(suf)) {
		    sufMap.put(suf.trim(),nn);
		    revSufMap.put(nn,suf.trim());
		    nn++;
		}
	    }
	}
	
	int guessed[] = new int[sufMap.size()];
	int real[] = new int[sufMap.size()];
	int correct[] = new int[sufMap.size()];
	
	int acc_corr = 0; int acc_tot = 0;
	
	System.out.println("Evaluating ... ");

	ObjectInputStream in = new ObjectInputStream(new FileInputStream(featureFile));

	for(int k = 0; k < data.length; k++) {

	    Instance inst = data[k];
	    Features feats = data[k].getFeatures(in);

	    SequencePrediction prediction = (SequencePrediction)predictor.decode(inst,feats);

	    String[] act_spans = ((SequenceLabel)inst.getLabel()).tags;
	    String[] pred_spans = ((SequenceLabel)prediction.getBestLabel()).tags;

	    for(int i = 0; i < pred_spans.length; i++) {
		String pred = pred_spans[i];
		String act = act_spans[i];
		if(pred.equals(act))
		    acc_corr++;
		acc_tot++;
		
		if(pred.startsWith("B")) {
		    String pred_suf = pred.split("-")[1];
		    guessed[sufMap.get(pred_suf)]++;
		    
		}
		if(act.startsWith("B")) {
		    String act_suf = act.split("-")[1];
		    real[sufMap.get(act_suf)]++;
		}				
		if(pred.startsWith("B") && act.startsWith("B") && pred.equals(act)) {
		    
		    String pred_suf = pred.split("-")[1];
		    
		    String cont = "I-"+pred_suf;
		    
		    for(int j = i+1; j < pred_spans.length; j++) {
			pred = pred_spans[j];
			act = act_spans[j];
			if(pred.equals(act) && pred.equals(cont) && j == pred_spans.length-1) {
			    correct[sufMap.get(pred_suf)]++;
			    break;
			}
			if(pred.equals(cont) && !act.equals(cont)) {
			    break;
			}
			if(!pred.equals(cont) && act.equals(cont))
			    break;
			if(!pred.equals(cont) && !act.equals(cont)) {
			    correct[sufMap.get(pred_suf)]++;
			    break;
			}
		    }
		    
		    if(i == pred_spans.length-1)
			correct[sufMap.get(pred_suf)]++;
		    
		}
		
	    }
	}
	
	in.close();
	DecimalFormat nf = new DecimalFormat();
	nf.setMaximumFractionDigits(5);
 
	int tot_corr = 0;
	int tot_guessed = 0;
	int tot_real = 0;
	for(int i = 0; i < correct.length; i++) {
	    System.out.println(revSufMap.get(i) + ":");
	    System.out.println("\tGuessed: " + guessed[i] + ", Actual: " 
			       + real[i] + ", Correct: " + correct[i]);
	    double prec = (double)correct[i]/guessed[i];
	    double rec = (double)correct[i]/real[i];
	    double fmeas = (2*prec*rec)/(prec+rec);
	    System.out.println("\tPrecision: " + nf.format(prec) 
			       + ", Recall: " + nf.format(rec) 
			       + ", F-meas: " + nf.format(fmeas));
	    tot_corr += correct[i];
	    tot_real += real[i];
	    tot_guessed += guessed[i];
	}
	System.out.println("Total:");
	System.out.println("\tGuessed: " + tot_guessed + ", Actual: " 
			   + tot_real + ", Correct: " + tot_corr);
	double prec = (double)tot_corr/tot_guessed;
	double rec = (double)tot_corr/tot_real;
	double fmeas = (2*prec*rec)/(prec+rec);
	System.out.println("\tPrecision: " + nf.format(prec)
			   + ", Recall: " + nf.format(rec) 
			   + ", F-meas: " + nf.format(fmeas));
	
	System.out.println("done.");
    }


}
